@@ -20,15 +20,15 @@
 
 /**
  * SECTION:element-assrender
+ * @title: assrender
  *
  * Renders timestamped SSA/ASS subtitles on top of a video stream.
  *
- * <refsect2>
- * <title>Example launch line</title>
+ * ## Example launch line
  * |[
- * gst-launch-1.0 -v filesrc location=/path/to/mkv ! matroskademux name=d ! queue ! mpegaudioparse ! mad ! audioconvert ! autoaudiosink  d. ! queue ! h264parse ! avdec_h264 ! videoconvert ! r.   d. ! queue ! "application/x-ass" ! assrender name=r ! videoconvert ! autovideosink
+ * gst-launch-1.0 -v filesrc location=/path/to/mkv ! matroskademux name=d ! queue ! mpegaudioparse ! mpg123audiodec ! audioconvert ! autoaudiosink  d. ! queue ! h264parse ! avdec_h264 ! videoconvert ! r.   d. ! queue ! "application/x-ass" ! assrender name=r ! videoconvert ! autovideosink
  * ]| This pipeline demuxes a Matroska file with h.264 video, MP3 audio and embedded ASS subtitles and renders the subtitles on top of the video.
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -930,12 +930,16 @@ gst_ass_render_negotiate (GstAssRender * render, GstCaps * caps)
   gst_caps_unref (overlay_caps);
   gst_caps_unref (caps);
 
+  if (!ret)
+    gst_pad_mark_reconfigure (render->srcpad);
+
   return ret;
 
 no_format:
   {
     if (caps)
       gst_caps_unref (caps);
+    gst_pad_mark_reconfigure (render->srcpad);
     return FALSE;
   }
 }
@@ -1160,8 +1164,15 @@ gst_ass_render_chain_video (GstPad * pad, GstObject * parent,
   guint64 start, stop, clip_start = 0, clip_stop = 0;
   ASS_Image *ass_image;
 
-  if (gst_pad_check_reconfigure (render->srcpad))
-    gst_ass_render_negotiate (render, NULL);
+  if (gst_pad_check_reconfigure (render->srcpad)) {
+    if (!gst_ass_render_negotiate (render, NULL)) {
+      gst_pad_mark_reconfigure (render->srcpad);
+      if (GST_PAD_IS_FLUSHING (render->srcpad))
+        goto flushing;
+      else
+        goto not_negotiated;
+    }
+  }
 
   if (!GST_BUFFER_TIMESTAMP_IS_VALID (buffer))
     goto missing_timestamp;
@@ -1379,6 +1390,13 @@ missing_timestamp:
     gst_buffer_unref (buffer);
     return GST_FLOW_OK;
   }
+not_negotiated:
+  {
+    GST_ASS_RENDER_UNLOCK (render);
+    GST_DEBUG_OBJECT (render, "not negotiated");
+    gst_buffer_unref (buffer);
+    return GST_FLOW_NOT_NEGOTIATED;
+  }
 flushing:
   {
     GST_ASS_RENDER_UNLOCK (render);
@@ -1695,7 +1713,6 @@ gst_ass_render_query_video (GstPad * pad, GstObject * parent, GstQuery * query)
 static gboolean
 gst_ass_render_event_text (GstPad * pad, GstObject * parent, GstEvent * event)
 {
-  gint i;
   gboolean ret = FALSE;
   GstAssRender *render = GST_ASS_RENDER (parent);
 
@@ -1766,6 +1783,11 @@ gst_ass_render_event_text (GstPad * pad, GstObject * parent, GstEvent * event)
       break;
     }
     case GST_EVENT_FLUSH_STOP:
+      g_mutex_lock (&render->ass_mutex);
+      if (render->ass_track) {
+        ass_flush_events (render->ass_track);
+      }
+      g_mutex_unlock (&render->ass_mutex);
       GST_ASS_RENDER_LOCK (render);
       GST_INFO_OBJECT (render, "text flush stop");
       render->subtitle_flushing = FALSE;
@@ -1778,17 +1800,6 @@ gst_ass_render_event_text (GstPad * pad, GstObject * parent, GstEvent * event)
       break;
     case GST_EVENT_FLUSH_START:
       GST_DEBUG_OBJECT (render, "text flush start");
-      g_mutex_lock (&render->ass_mutex);
-      if (render->ass_track) {
-        /* delete any events on the ass_track */
-        for (i = 0; i < render->ass_track->n_events; i++) {
-          GST_DEBUG_OBJECT (render, "deleted event with eid %i", i);
-          ass_free_event (render->ass_track, i);
-        }
-        render->ass_track->n_events = 0;
-        GST_DEBUG_OBJECT (render, "done flushing");
-      }
-      g_mutex_unlock (&render->ass_mutex);
       GST_ASS_RENDER_LOCK (render);
       render->subtitle_flushing = TRUE;
       GST_ASS_RENDER_BROADCAST (render);
