@@ -168,6 +168,14 @@ gst_dtls_agent_class_init (GstDtlsAgentClass * klass)
   _gst_dtls_init_openssl ();
 }
 
+static int
+ssl_warn_cb (const char *str, size_t len, void *u)
+{
+  GstDtlsAgent *self = u;
+  GST_WARNING_OBJECT (self, "ssl error: %s", str);
+  return 0;
+}
+
 static void
 gst_dtls_agent_init (GstDtlsAgent * self)
 {
@@ -181,16 +189,17 @@ gst_dtls_agent_init (GstDtlsAgent * self)
 #else
   priv->ssl_context = SSL_CTX_new (DTLSv1_method ());
 #endif
-  if (ERR_peek_error () || !priv->ssl_context) {
-    char buf[512];
-
-    priv->ssl_context = NULL;
-
-    GST_WARNING_OBJECT (self, "Error creating SSL Context: %s",
-        ERR_error_string (ERR_get_error (), buf));
+  if (!priv->ssl_context) {
+    GST_WARNING_OBJECT (self, "Error creating SSL Context");
+    ERR_print_errors_cb (ssl_warn_cb, self);
 
     g_return_if_reached ();
   }
+  /* If any non-fatal issues happened, print them out and carry on */
+  if (ERR_peek_error ()) {
+    ERR_print_errors_cb (ssl_warn_cb, self);
+    ERR_clear_error ();
+  }
 
   SSL_CTX_set_verify_depth (priv->ssl_context, 2);
   SSL_CTX_set_tlsext_use_srtp (priv->ssl_context, "SRTP_AES128_CM_SHA1_80");
