@@ -26,15 +26,15 @@
  */
 /**
  * SECTION:element-hlsdemux
+ * @title: hlsdemux
  *
  * HTTP Live Streaming demuxer element.
  *
- * <refsect2>
- * <title>Example launch line</title>
+ * ## Example launch line
  * |[
  * gst-launch-1.0 souphttpsrc location=http://devimages.apple.com/iphone/samples/bipbop/gear4/prog_index.m3u8 ! hlsdemux ! decodebin ! videoconvert ! videoscale ! autovideosink
  * ]|
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -90,6 +90,9 @@ static gboolean gst_hls_demux_process_manifest (GstAdaptiveDemux * demux,
     GstBuffer * buf);
 static GstFlowReturn gst_hls_demux_update_manifest (GstAdaptiveDemux * demux);
 static gboolean gst_hls_demux_seek (GstAdaptiveDemux * demux, GstEvent * seek);
+static GstFlowReturn gst_hls_demux_stream_seek (GstAdaptiveDemuxStream *
+    stream, gboolean forward, GstSeekFlags flags, GstClockTime ts,
+    GstClockTime * final_ts);
 static gboolean
 gst_hls_demux_start_fragment (GstAdaptiveDemux * demux,
     GstAdaptiveDemuxStream * stream);
@@ -165,6 +168,7 @@ gst_hls_demux_class_init (GstHLSDemuxClass * klass)
   adaptivedemux_class->update_manifest = gst_hls_demux_update_manifest;
   adaptivedemux_class->reset = gst_hls_demux_reset;
   adaptivedemux_class->seek = gst_hls_demux_seek;
+  adaptivedemux_class->stream_seek = gst_hls_demux_stream_seek;
   adaptivedemux_class->stream_has_next_fragment =
       gst_hls_demux_stream_has_next_fragment;
   adaptivedemux_class->stream_advance_fragment = gst_hls_demux_advance_fragment;
@@ -294,34 +298,41 @@ gst_hls_demux_set_current (GstHLSDemux * self, GstM3U8 * m3u8)
 }
 #endif
 
+#define SEEK_UPDATES_PLAY_POSITION(r, start_type, stop_type) \
+  ((r >= 0 && start_type != GST_SEEK_TYPE_NONE) || \
+   (r < 0 && stop_type != GST_SEEK_TYPE_NONE))
+
+#define IS_SNAP_SEEK(f) (f & (GST_SEEK_FLAG_SNAP_BEFORE |	  \
+                              GST_SEEK_FLAG_SNAP_AFTER |	  \
+                              GST_SEEK_FLAG_SNAP_NEAREST |	  \
+			      GST_SEEK_FLAG_TRICKMODE_KEY_UNITS | \
+			      GST_SEEK_FLAG_KEY_UNIT))
+
 static gboolean
 gst_hls_demux_seek (GstAdaptiveDemux * demux, GstEvent * seek)
 {
   GstHLSDemux *hlsdemux = GST_HLS_DEMUX_CAST (demux);
   GstFormat format;
   GstSeekFlags flags;
-  GstSeekType start_type, stop_type, target_type;
+  GstSeekType start_type, stop_type;
   gint64 start, stop;
   gdouble rate, old_rate;
-  GList *walk, *stream_walk;
-  GstClockTime current_pos, target_pos;
-  gint64 current_sequence;
+  GList *walk;
+  GstClockTime current_pos, target_pos, final_pos;
   guint64 bitrate;
-  gboolean snap_before, snap_after, snap_nearest, keyunit;
-  gboolean reverse;
-
-  old_rate = demux->segment.rate;
 
   gst_event_parse_seek (seek, &rate, &format, &flags, &start_type, &start,
       &stop_type, &stop);
 
-  bitrate = gst_hls_demux_get_bitrate (hlsdemux);
-
-  /* properly cleanup pending decryption status */
-  if (flags & GST_SEEK_FLAG_FLUSH) {
-    gst_hls_demux_clear_all_pending_data (hlsdemux);
+  if (!SEEK_UPDATES_PLAY_POSITION (rate, start_type, stop_type)) {
+    /* nothing to do if we don't have to update the current position */
+    return TRUE;
   }
 
+  old_rate = demux->segment.rate;
+
+  bitrate = gst_hls_demux_get_bitrate (hlsdemux);
+
   /* Use I-frame variants for trick modes */
   if (hlsdemux->master->iframe_variants != NULL
       && rate < -1.0 && old_rate >= -1.0 && old_rate <= 1.0) {
@@ -353,89 +364,111 @@ gst_hls_demux_seek (GstAdaptiveDemux * demux, GstEvent * seek)
     gst_hls_demux_change_playlist (hlsdemux, bitrate, NULL);
   }
 
-  for (stream_walk = demux->streams; stream_walk != NULL;
-      stream_walk = stream_walk->next) {
-    GstHLSDemuxStream *hls_stream =
-        GST_HLS_DEMUX_STREAM_CAST (stream_walk->data);
-    GstM3U8MediaFile *file = NULL;
+  target_pos = rate < 0 ? stop : start;
+  final_pos = target_pos;
 
-    current_sequence = 0;
-    current_pos = 0;
-    reverse = rate < 0;
-    target_pos = reverse ? stop : start;
-    target_type = reverse ? stop_type : start_type;
+  /* properly cleanup pending decryption status */
+  if (flags & GST_SEEK_FLAG_FLUSH) {
+    gst_hls_demux_clear_all_pending_data (hlsdemux);
+  }
 
-    if (target_type == GST_SEEK_TYPE_NONE && !(flags & GST_SEEK_FLAG_FLUSH)) {
-      /* No need to move */
+  for (walk = demux->streams; walk; walk = g_list_next (walk)) {
+    GstAdaptiveDemuxStream *stream =
+        GST_ADAPTIVE_DEMUX_STREAM_CAST (walk->data);
+
+    gst_hls_demux_stream_seek (stream, rate >= 0, flags, target_pos,
+        &current_pos);
+
+    /* FIXME: use minimum position always ? */
+    if (final_pos > current_pos)
+      final_pos = current_pos;
+  }
+
+  if (IS_SNAP_SEEK (flags)) {
+    if (rate >= 0)
       gst_segment_do_seek (&demux->segment, rate, format, flags, start_type,
-          start, stop_type, stop, NULL);
-    } else {
-      /* Snap to segment boundary. Improves seek performance on slow machines. */
-      keyunit = ! !(flags & GST_SEEK_FLAG_KEY_UNIT);
-      snap_nearest =
-          (flags & GST_SEEK_FLAG_SNAP_NEAREST) == GST_SEEK_FLAG_SNAP_NEAREST;
-      snap_before = ! !(flags & GST_SEEK_FLAG_SNAP_BEFORE);
-      snap_after = ! !(flags & GST_SEEK_FLAG_SNAP_AFTER);
-
-      GST_M3U8_CLIENT_LOCK (hlsdemux->client);
-      /* FIXME: Here we need proper discont handling */
-      for (walk = hls_stream->playlist->files; walk; walk = walk->next) {
-        file = walk->data;
-
-        current_sequence = file->sequence;
-        if ((!reverse && snap_after) || snap_nearest) {
-          if (current_pos >= target_pos)
-            break;
-          if (snap_nearest && target_pos - current_pos < file->duration / 2)
-            break;
-        } else if (reverse && snap_after) {
-          /* check if the next fragment is our target, in this case we want to
-           * start from the previous fragment */
-          GstClockTime next_pos = current_pos + file->duration;
-
-          if (next_pos <= target_pos && target_pos < next_pos + file->duration) {
-            break;
-          }
-        } else if (current_pos <= target_pos
-            && target_pos < current_pos + file->duration) {
-          break;
-        }
-        current_pos += file->duration;
-      }
-
-      if (walk == NULL) {
-        GST_DEBUG_OBJECT (demux, "seeking further than track duration");
-        current_sequence++;
-      }
-
-      GST_DEBUG_OBJECT (demux, "seeking to sequence %u",
-          (guint) current_sequence);
-      hls_stream->reset_pts = TRUE;
-      hls_stream->playlist->sequence = current_sequence;
-      hls_stream->playlist->current_file = walk;
-      hls_stream->playlist->sequence_position = current_pos;
-      GST_M3U8_CLIENT_UNLOCK (hlsdemux->client);
-
-      /* Play from the end of the current selected segment */
-      if (file) {
-        if (reverse && (snap_before || snap_after || snap_nearest))
-          current_pos += file->duration;
-      }
-
-      if (keyunit || snap_before || snap_after || snap_nearest) {
-        if (!reverse)
-          gst_segment_do_seek (&demux->segment, rate, format, flags, start_type,
-              current_pos, stop_type, stop, NULL);
-        else
-          gst_segment_do_seek (&demux->segment, rate, format, flags, start_type,
-              start, stop_type, current_pos, NULL);
-      }
-    }
+          final_pos, stop_type, stop, NULL);
+    else
+      gst_segment_do_seek (&demux->segment, rate, format, flags, start_type,
+          start, stop_type, final_pos, NULL);
   }
 
   return TRUE;
 }
 
+static GstFlowReturn
+gst_hls_demux_stream_seek (GstAdaptiveDemuxStream * stream, gboolean forward,
+    GstSeekFlags flags, GstClockTime ts, GstClockTime * final_ts)
+{
+  GstHLSDemuxStream *hls_stream = GST_HLS_DEMUX_STREAM_CAST (stream);
+  GList *walk;
+  GstClockTime current_pos;
+  gint64 current_sequence;
+  gboolean snap_after, snap_nearest;
+  GstM3U8MediaFile *file = NULL;
+
+  current_sequence = 0;
+  current_pos = gst_m3u8_is_live (hls_stream->playlist) ?
+      hls_stream->playlist->first_file_start : 0;
+
+  /* Snap to segment boundary. Improves seek performance on slow machines. */
+  snap_nearest =
+      (flags & GST_SEEK_FLAG_SNAP_NEAREST) == GST_SEEK_FLAG_SNAP_NEAREST;
+  snap_after = ! !(flags & GST_SEEK_FLAG_SNAP_AFTER);
+
+  GST_M3U8_CLIENT_LOCK (hlsdemux->client);
+  /* FIXME: Here we need proper discont handling */
+  for (walk = hls_stream->playlist->files; walk; walk = walk->next) {
+    file = walk->data;
+
+    current_sequence = file->sequence;
+    if ((forward && snap_after) || snap_nearest) {
+      if (current_pos >= ts)
+        break;
+      if (snap_nearest && ts - current_pos < file->duration / 2)
+        break;
+    } else if (!forward && snap_after) {
+      /* check if the next fragment is our target, in this case we want to
+       * start from the previous fragment */
+      GstClockTime next_pos = current_pos + file->duration;
+
+      if (next_pos <= ts && ts < next_pos + file->duration) {
+        break;
+      }
+    } else if (current_pos <= ts && ts < current_pos + file->duration) {
+      break;
+    }
+    current_pos += file->duration;
+  }
+
+  if (walk == NULL) {
+    GST_DEBUG_OBJECT (stream->pad, "seeking further than track duration");
+    current_sequence++;
+  }
+
+  GST_DEBUG_OBJECT (stream->pad, "seeking to sequence %u",
+      (guint) current_sequence);
+  hls_stream->reset_pts = TRUE;
+  hls_stream->playlist->sequence = current_sequence;
+  hls_stream->playlist->current_file = walk;
+  hls_stream->playlist->sequence_position = current_pos;
+  GST_M3U8_CLIENT_UNLOCK (hlsdemux->client);
+
+  /* Play from the end of the current selected segment */
+  if (file) {
+    if (!forward && IS_SNAP_SEEK (flags))
+      current_pos += file->duration;
+  }
+
+  /* update stream's segment position */
+  stream->segment.position = current_pos;
+
+  if (final_ts)
+    *final_ts = current_pos;
+
+  return GST_FLOW_OK;
+}
+
 static GstFlowReturn
 gst_hls_demux_update_manifest (GstAdaptiveDemux * demux)
 {
@@ -769,19 +802,17 @@ gst_hls_demux_handle_buffer (GstAdaptiveDemux * demux,
 {
   GstHLSDemuxStream *hls_stream = GST_HLS_DEMUX_STREAM_CAST (stream);   // FIXME: pass HlsStream into function
   GstHLSDemux *hlsdemux = GST_HLS_DEMUX_CAST (demux);
-  GstMapInfo info;
   GstClockTime first_pcr, last_pcr;
   GstTagList *tags;
 
   if (buffer == NULL)
     return GST_FLOW_OK;
 
-  gst_buffer_map (buffer, &info, GST_MAP_READ);
-
   if (G_UNLIKELY (hls_stream->do_typefind)) {
     GstCaps *caps = NULL;
     guint buffer_size;
     GstTypeFindProbability prob = GST_TYPE_FIND_NONE;
+    GstMapInfo info;
 
     if (hls_stream->pending_typefind_buffer)
       buffer = gst_buffer_append (hls_stream->pending_typefind_buffer, buffer);
@@ -826,11 +857,11 @@ gst_hls_demux_handle_buffer (GstAdaptiveDemux * demux,
     gst_adaptive_demux_stream_set_caps (stream, caps);
 
     hls_stream->do_typefind = FALSE;
+
+    gst_buffer_unmap (buffer, &info);
   }
   g_assert (hls_stream->pending_typefind_buffer == NULL);
 
-  gst_buffer_unmap (buffer, &info);
-
   // Accumulate this buffer
   if (hls_stream->pending_pcr_buffer) {
     buffer = gst_buffer_append (hls_stream->pending_pcr_buffer, buffer);
@@ -847,6 +878,9 @@ gst_hls_demux_handle_buffer (GstAdaptiveDemux * demux,
 
   if (tags) {
     gst_adaptive_demux_stream_set_tags (stream, tags);
+    /* run typefind again on the trimmed buffer */
+    hls_stream->do_typefind = TRUE;
+    return gst_hls_demux_handle_buffer (demux, stream, buffer, at_eos);
   }
 
   if (buffer) {
@@ -892,6 +926,13 @@ gst_hls_demux_finish_fragment (GstAdaptiveDemux * demux,
     }
 
     if (ret == GST_FLOW_OK || ret == GST_FLOW_NOT_LINKED) {
+      if (G_UNLIKELY (hls_stream->pending_typefind_buffer)) {
+        GstBuffer *buf = hls_stream->pending_typefind_buffer;
+        hls_stream->pending_typefind_buffer = NULL;
+
+        gst_hls_demux_handle_buffer (demux, stream, buf, TRUE);
+      }
+
       if (hls_stream->pending_pcr_buffer) {
         GstBuffer *buf = hls_stream->pending_pcr_buffer;
         hls_stream->pending_pcr_buffer = NULL;
@@ -1078,6 +1119,8 @@ gst_hls_demux_update_fragment_info (GstAdaptiveDemuxStream * stream)
   if (discont)
     stream->discont = TRUE;
 
+  gst_m3u8_media_file_unref (file);
+
   return GST_FLOW_OK;
 }
 
@@ -1201,6 +1244,7 @@ gst_hls_demux_update_variant_playlist (GstHLSDemux * hlsdemux, gchar * data,
   if (hlsdemux->master->is_simple) {
     GST_ERROR
         ("Cannot update variant playlist: Current playlist is not a variant playlist");
+    gst_hls_master_playlist_unref (new_master);
     goto out;
   }
 
@@ -1457,7 +1501,7 @@ retry:
         "sequence:%" G_GINT64_FORMAT " , first_sequence:%" G_GINT64_FORMAT
         " , last_sequence:%" G_GINT64_FORMAT, m3u8->sequence,
         first_sequence, last_sequence);
-    if (m3u8->sequence >= last_sequence - 3) {
+    if (m3u8->sequence > last_sequence - 3) {
       //demux->need_segment = TRUE;
       /* Make sure we never go below the minimum sequence number */
       m3u8->sequence = MAX (first_sequence, last_sequence - 3);
@@ -1611,11 +1655,17 @@ static gboolean
 gst_hls_demux_stream_decrypt_start (GstHLSDemuxStream * stream,
     const guint8 * key_data, const guint8 * iv_data)
 {
+  EVP_CIPHER_CTX *ctx;
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
   EVP_CIPHER_CTX_init (&stream->aes_ctx);
-  if (!EVP_DecryptInit_ex (&stream->aes_ctx, EVP_aes_128_cbc (), NULL, key_data,
-          iv_data))
+  ctx = &stream->aes_ctx;
+#else
+  stream->aes_ctx = EVP_CIPHER_CTX_new ();
+  ctx = stream->aes_ctx;
+#endif
+  if (!EVP_DecryptInit_ex (ctx, EVP_aes_128_cbc (), NULL, key_data, iv_data))
     return FALSE;
-  EVP_CIPHER_CTX_set_padding (&stream->aes_ctx, 0);
+  EVP_CIPHER_CTX_set_padding (ctx, 0);
   return TRUE;
 }
 
@@ -1624,15 +1674,21 @@ decrypt_fragment (GstHLSDemuxStream * stream, gsize length,
     const guint8 * encrypted_data, guint8 * decrypted_data)
 {
   int len, flen = 0;
+  EVP_CIPHER_CTX *ctx;
+
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
+  ctx = &stream->aes_ctx;
+#else
+  ctx = stream->aes_ctx;
+#endif
 
   if (G_UNLIKELY (length > G_MAXINT || length % 16 != 0))
     return FALSE;
 
   len = (int) length;
-  if (!EVP_DecryptUpdate (&stream->aes_ctx, decrypted_data, &len,
-          encrypted_data, len))
+  if (!EVP_DecryptUpdate (ctx, decrypted_data, &len, encrypted_data, len))
     return FALSE;
-  EVP_DecryptFinal_ex (&stream->aes_ctx, decrypted_data + len, &flen);
+  EVP_DecryptFinal_ex (ctx, decrypted_data + len, &flen);
   g_return_val_if_fail (len + flen == length, FALSE);
   return TRUE;
 }
@@ -1640,7 +1696,12 @@ decrypt_fragment (GstHLSDemuxStream * stream, gsize length,
 static void
 gst_hls_demux_stream_decrypt_end (GstHLSDemuxStream * stream)
 {
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
   EVP_CIPHER_CTX_cleanup (&stream->aes_ctx);
+#else
+  EVP_CIPHER_CTX_free (stream->aes_ctx);
+  stream->aes_ctx = NULL;
+#endif
 }
 
 #elif defined(HAVE_NETTLE)
