@@ -60,6 +60,8 @@ gst_vulkan_device_new (GstVulkanInstance * instance)
 {
   GstVulkanDevice *device = g_object_new (GST_TYPE_VULKAN_DEVICE, NULL);
 
+  gst_object_ref_sink (device);
+
   device->instance = gst_object_ref (instance);
   /* FIXME: select this externally */
   device->device_index = 0;
@@ -230,7 +232,7 @@ gst_vulkan_device_open (GstVulkanDevice * device, GError ** error)
     goto error;
   }
 
-  for (uint32_t i = 0; i < device_extension_count; i++) {
+  for (i = 0; i < device_extension_count; i++) {
     GST_TRACE_OBJECT (device, "checking device extension %s",
         device_extensions[i].extensionName);
     if (!strcmp (VK_KHR_SWAPCHAIN_EXTENSION_NAME,
@@ -350,6 +352,7 @@ gst_vulkan_device_get_queue (GstVulkanDevice * device, guint32 queue_family,
       device->queue_family_props[queue_family].queueCount, NULL);
 
   ret = g_object_new (GST_TYPE_VULKAN_QUEUE, NULL);
+  gst_object_ref_sink (ret);
   ret->device = gst_object_ref (device);
   ret->family = queue_family;
   ret->index = queue_i;
