@@ -46,11 +46,21 @@ static void
 handle_configure (void *data, struct wl_shell_surface *shell_surface,
     uint32_t edges, int32_t width, int32_t height)
 {
+  GstWlWindow *window = data;
+
+  GST_DEBUG ("Windows configure: edges %x, width = %i, height %i", edges,
+      width, height);
+
+  if (width == 0 || height == 0)
+    return;
+
+  gst_wl_window_set_render_rectangle (window, 0, 0, width, height);
 }
 
 static void
 handle_popup_done (void *data, struct wl_shell_surface *shell_surface)
 {
+  GST_DEBUG ("Window popup done.");
 }
 
 static const struct wl_shell_surface_listener shell_surface_listener = {
@@ -76,18 +86,21 @@ gst_wl_window_finalize (GObject * gobject)
 {
   GstWlWindow *self = GST_WL_WINDOW (gobject);
 
-  if (self->shell_surface) {
+  if (self->shell_surface)
     wl_shell_surface_destroy (self->shell_surface);
-  }
 
-  wl_viewport_destroy (self->video_viewport);
+  if (self->video_viewport)
+    wp_viewport_destroy (self->video_viewport);
+
   wl_subsurface_destroy (self->video_subsurface);
   wl_surface_destroy (self->video_surface);
 
-  if (self->area_subsurface) {
+  if (self->area_subsurface)
     wl_subsurface_destroy (self->area_subsurface);
-  }
-  wl_viewport_destroy (self->area_viewport);
+
+  if (self->area_viewport)
+    wp_viewport_destroy (self->area_viewport);
+
   wl_surface_destroy (self->area_surface);
 
   g_clear_object (&self->display);
@@ -96,18 +109,13 @@ gst_wl_window_finalize (GObject * gobject)
 }
 
 static GstWlWindow *
-gst_wl_window_new_internal (GstWlDisplay * display)
+gst_wl_window_new_internal (GstWlDisplay * display, GMutex * render_lock)
 {
   GstWlWindow *window;
-  GstVideoInfo info;
-  GstBuffer *buf;
-  GstMapInfo mapinfo;
-  struct wl_buffer *wlbuf;
-  GstWlBuffer *gwlbuf;
-  struct wl_region *region;
 
   window = g_object_new (GST_TYPE_WL_WINDOW, NULL);
   window->display = g_object_ref (display);
+  window->render_lock = render_lock;
 
   window->area_surface = wl_compositor_create_surface (display->compositor);
   window->video_surface = wl_compositor_create_surface (display->compositor);
@@ -122,54 +130,28 @@ gst_wl_window_new_internal (GstWlDisplay * display)
       window->video_surface, window->area_surface);
   wl_subsurface_set_desync (window->video_subsurface);
 
-  window->area_viewport = wl_scaler_get_viewport (display->scaler,
-      window->area_surface);
-  window->video_viewport = wl_scaler_get_viewport (display->scaler,
-      window->video_surface);
-
-  /* draw the area_subsurface */
-  gst_video_info_set_format (&info,
-      /* we want WL_SHM_FORMAT_XRGB8888 */
-#if G_BYTE_ORDER == G_BIG_ENDIAN
-      GST_VIDEO_FORMAT_xRGB,
-#else
-      GST_VIDEO_FORMAT_BGRx,
-#endif
-      1, 1);
-
-  buf = gst_buffer_new_allocate (gst_wl_shm_allocator_get (), info.size, NULL);
-  gst_buffer_map (buf, &mapinfo, GST_MAP_WRITE);
-  *((guint32 *) mapinfo.data) = 0;      /* paint it black */
-  gst_buffer_unmap (buf, &mapinfo);
-  wlbuf =
-      gst_wl_shm_memory_construct_wl_buffer (gst_buffer_peek_memory (buf, 0),
-      display, &info);
-  gwlbuf = gst_buffer_add_wl_buffer (buf, wlbuf, display);
-  gst_wl_buffer_attach (gwlbuf, window->area_surface);
-
-  /* at this point, the GstWlBuffer keeps the buffer
-   * alive and will free it on wl_buffer::release */
-  gst_buffer_unref (buf);
+  if (display->viewporter) {
+    window->area_viewport = wp_viewporter_get_viewport (display->viewporter,
+        window->area_surface);
+    window->video_viewport = wp_viewporter_get_viewport (display->viewporter,
+        window->video_surface);
+  }
 
   /* do not accept input */
-  region = wl_compositor_create_region (display->compositor);
-  wl_surface_set_input_region (window->area_surface, region);
-  wl_region_destroy (region);
-
-  region = wl_compositor_create_region (display->compositor);
-  wl_surface_set_input_region (window->video_surface, region);
-  wl_region_destroy (region);
+  wl_surface_set_input_region (window->area_surface, NULL);
+  wl_surface_set_input_region (window->video_surface, NULL);
 
   return window;
 }
 
 GstWlWindow *
-gst_wl_window_new_toplevel (GstWlDisplay * display, const GstVideoInfo * info)
+gst_wl_window_new_toplevel (GstWlDisplay * display, const GstVideoInfo * info,
+    GMutex * render_lock)
 {
   GstWlWindow *window;
   gint width;
 
-  window = gst_wl_window_new_internal (display);
+  window = gst_wl_window_new_internal (display, render_lock);
 
   /* go toplevel */
   window->shell_surface = wl_shell_get_shell_surface (display->shell,
@@ -196,10 +178,10 @@ gst_wl_window_new_toplevel (GstWlDisplay * display, const GstVideoInfo * info)
 
 GstWlWindow *
 gst_wl_window_new_in_surface (GstWlDisplay * display,
-    struct wl_surface * parent)
+    struct wl_surface * parent, GMutex * render_lock)
 {
   GstWlWindow *window;
-  window = gst_wl_window_new_internal (display);
+  window = gst_wl_window_new_internal (display, render_lock);
 
   /* embed in parent */
   window->area_subsurface =
@@ -246,10 +228,15 @@ gst_wl_window_resize_video_surface (GstWlWindow * window, gboolean commit)
   src.h = window->video_height;
   dst.w = window->render_rectangle.w;
   dst.h = window->render_rectangle.h;
-  gst_video_sink_center_rect (src, dst, &res, TRUE);
+
+  if (window->video_viewport) {
+    gst_video_sink_center_rect (src, dst, &res, TRUE);
+    wp_viewport_set_destination (window->video_viewport, res.w, res.h);
+  } else {
+    gst_video_sink_center_rect (src, dst, &res, FALSE);
+  }
 
   wl_subsurface_set_position (window->video_subsurface, res.x, res.y);
-  wl_viewport_set_destination (window->video_viewport, res.w, res.h);
 
   if (commit) {
     wl_surface_damage (window->video_surface, 0, 0, res.w, res.h);
@@ -267,8 +254,29 @@ gst_wl_window_resize_video_surface (GstWlWindow * window, gboolean commit)
   }
 
   /* this is saved for use in wl_surface_damage */
-  window->surface_width = res.w;
-  window->surface_height = res.h;
+  window->video_rectangle = res;
+}
+
+static void
+gst_wl_window_set_opaque (GstWlWindow * window, const GstVideoInfo * info)
+{
+  struct wl_region *region;
+
+  /* Set area opaque */
+  region = wl_compositor_create_region (window->display->compositor);
+  wl_region_add (region, 0, 0, window->render_rectangle.w,
+      window->render_rectangle.h);
+  wl_surface_set_opaque_region (window->area_surface, region);
+  wl_region_destroy (region);
+
+  if (!GST_VIDEO_INFO_HAS_ALPHA (info)) {
+    /* Set video opaque */
+    region = wl_compositor_create_region (window->display->compositor);
+    wl_region_add (region, 0, 0, window->render_rectangle.w,
+        window->render_rectangle.h);
+    wl_surface_set_opaque_region (window->video_surface, region);
+    wl_region_destroy (region);
+  }
 }
 
 void
@@ -282,6 +290,7 @@ gst_wl_window_render (GstWlWindow * window, GstWlBuffer * buffer,
 
     wl_subsurface_set_sync (window->video_subsurface);
     gst_wl_window_resize_video_surface (window, FALSE);
+    gst_wl_window_set_opaque (window, info);
   }
 
   if (G_LIKELY (buffer))
@@ -289,8 +298,8 @@ gst_wl_window_render (GstWlWindow * window, GstWlBuffer * buffer,
   else
     wl_surface_attach (window->video_surface, NULL, 0, 0);
 
-  wl_surface_damage (window->video_surface, 0, 0, window->surface_width,
-      window->surface_height);
+  wl_surface_damage (window->video_surface, 0, 0, window->video_rectangle.w,
+      window->video_rectangle.h);
   wl_surface_commit (window->video_surface);
 
   if (G_UNLIKELY (info)) {
@@ -305,6 +314,57 @@ gst_wl_window_render (GstWlWindow * window, GstWlBuffer * buffer,
   wl_display_flush (window->display->display);
 }
 
+/* Update the buffer used to draw black borders. When we have viewporter
+ * support, this is a scaled up 1x1 image, and without we need an black image
+ * the size of the rendering areay. */
+static void
+gst_wl_window_update_borders (GstWlWindow * window)
+{
+  GstVideoFormat format;
+  GstVideoInfo info;
+  gint width, height;
+  GstBuffer *buf;
+  struct wl_buffer *wlbuf;
+  GstWlBuffer *gwlbuf;
+  GstAllocator *alloc;
+
+  if (window->no_border_update)
+    return;
+
+  if (window->display->viewporter) {
+    width = height = 1;
+    window->no_border_update = TRUE;
+  } else {
+    width = window->render_rectangle.w;
+    height = window->render_rectangle.h;
+  }
+
+  /* we want WL_SHM_FORMAT_XRGB8888 */
+#if G_BYTE_ORDER == G_BIG_ENDIAN
+  format = GST_VIDEO_FORMAT_xRGB;
+#else
+  format = GST_VIDEO_FORMAT_BGRx;
+#endif
+
+  /* draw the area_subsurface */
+  gst_video_info_set_format (&info, format, width, height);
+
+  alloc = gst_wl_shm_allocator_get ();
+
+  buf = gst_buffer_new_allocate (alloc, info.size, NULL);
+  gst_buffer_memset (buf, 0, 0, info.size);
+  wlbuf =
+      gst_wl_shm_memory_construct_wl_buffer (gst_buffer_peek_memory (buf, 0),
+      window->display, &info);
+  gwlbuf = gst_buffer_add_wl_buffer (buf, wlbuf, window->display);
+  gst_wl_buffer_attach (gwlbuf, window->area_surface);
+
+  /* at this point, the GstWlBuffer keeps the buffer
+   * alive and will free it on wl_buffer::release */
+  gst_buffer_unref (buf);
+  g_object_unref (alloc);
+}
+
 void
 gst_wl_window_set_render_rectangle (GstWlWindow * window, gint x, gint y,
     gint w, gint h)
@@ -321,7 +381,10 @@ gst_wl_window_set_render_rectangle (GstWlWindow * window, gint x, gint y,
     wl_subsurface_set_position (window->area_subsurface, x, y);
 
   /* change the size of the area */
-  wl_viewport_set_destination (window->area_viewport, w, h);
+  if (window->area_viewport)
+    wp_viewport_set_destination (window->area_viewport, w, h);
+
+  gst_wl_window_update_borders (window);
 
   if (window->video_width != 0) {
     wl_subsurface_set_sync (window->video_subsurface);
