@@ -77,7 +77,7 @@ nal_reader_init (NalReader * nr, const guint8 * data, guint size)
   nr->cache = 0xff;
 }
 
-extern inline gboolean
+gboolean
 nal_reader_read (NalReader * nr, guint nbits)
 {
   if (G_UNLIKELY (nr->byte * 8 + (nbits - nr->bits_in_cache) > nr->size * 8)) {
@@ -115,7 +115,7 @@ nal_reader_read (NalReader * nr, guint nbits)
 
 /* Skips the specified amount of bits. This is only suitable to a
    cacheable number of bits */
-extern inline gboolean
+gboolean
 nal_reader_skip (NalReader * nr, guint nbits)
 {
   g_assert (nbits <= 8 * sizeof (nr->cache));
@@ -146,19 +146,19 @@ nal_reader_skip_long (NalReader * nr, guint nbits)
   return TRUE;
 }
 
-extern inline guint
+guint
 nal_reader_get_pos (const NalReader * nr)
 {
   return nr->byte * 8 - nr->bits_in_cache;
 }
 
-extern inline guint
+guint
 nal_reader_get_remaining (const NalReader * nr)
 {
   return (nr->size - nr->byte) * 8 + nr->bits_in_cache;
 }
 
-extern inline guint
+guint
 nal_reader_get_epb_count (const NalReader * nr)
 {
   return nr->n_epb;
@@ -219,7 +219,7 @@ nal_reader_get_ue (NalReader * nr, guint32 * val)
       return FALSE;
   }
 
-  if (G_UNLIKELY (i > 32))
+  if (G_UNLIKELY (i > 31))
     return FALSE;
 
   if (G_UNLIKELY (!nal_reader_get_bits_uint32 (nr, &value, i)))
@@ -230,7 +230,7 @@ nal_reader_get_ue (NalReader * nr, guint32 * val)
   return TRUE;
 }
 
-extern inline gboolean
+gboolean
 nal_reader_get_se (NalReader * nr, gint32 * val)
 {
   guint32 value;
@@ -297,7 +297,7 @@ nal_reader_has_more_data (NalReader * nr)
 
 /***********  end of nal parser ***************/
 
-extern inline gint
+gint
 scan_for_start_codes (const guint8 * data, guint size)
 {
   GstByteReader br;
