@@ -354,6 +354,16 @@ struct _GstMpegtsLogicalChannelDescriptor
   GstMpegtsLogicalChannel channels[64];
 };
 
+#define GST_TYPE_MPEGTS_LOGICAL_CHANNEL_DESCRIPTOR (gst_mpegts_logical_channel_descriptor_get_type())
+
+GST_MPEGTS_API
+GType gst_mpegts_logical_channel_descriptor_get_type(void);
+
+#define GST_TYPE_MPEGTS_LOGICAL_CHANNEL (gst_mpegts_logical_channel_get_type())
+
+GST_MPEGTS_API
+GType gst_mpegts_logical_channel_get_type(void);
+
 /* FIXME : Maybe make two methods. One for getting the number of channels,
  * and the other for getting the content for one channel ? */
 GST_MPEGTS_API
