@@ -421,6 +421,8 @@ adpcmenc_stop (GstAudioEncoder * enc)
 static void
 adpcmenc_init (ADPCMEnc * enc)
 {
+  GST_PAD_SET_ACCEPT_TEMPLATE (GST_AUDIO_ENCODER_SINK_PAD (enc));
+
   /* Set defaults. */
   enc->blocksize = DEFAULT_ADPCM_BLOCK_SIZE;
   enc->layout = DEFAULT_ADPCM_LAYOUT;
@@ -436,13 +438,12 @@ adpcmenc_class_init (ADPCMEncClass * klass)
   gobjectclass->set_property = adpcmenc_set_property;
   gobjectclass->get_property = adpcmenc_get_property;
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&adpcmenc_sink_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&adpcmenc_src_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &adpcmenc_sink_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &adpcmenc_src_template);
   gst_element_class_set_static_metadata (element_class, "ADPCM encoder",
-      "Codec/Encoder/Audio",
-      "Encode ADPCM audio",
+      "Codec/Encoder/Audio", "Encode ADPCM audio",
       "Pioneers of the Inevitable <songbird@songbirdnest.com>");
 
   base_class->start = GST_DEBUG_FUNCPTR (adpcmenc_start);
