@@ -228,6 +228,20 @@ tsmux_get_pat_interval (TsMux * mux)
   return mux->pat_interval;
 }
 
+/**
+ * tsmux_resend_pat:
+ * @mux: a #TsMux
+ *
+ * Resends the PAT before the next stream packet.
+ */
+void
+tsmux_resend_pat (TsMux * mux)
+{
+  g_return_if_fail (mux != NULL);
+
+  mux->last_pat_ts = G_MININT64;
+}
+
 /**
  * tsmux_set_si_interval:
  * @mux: a #TsMux
@@ -260,6 +274,21 @@ tsmux_get_si_interval (TsMux * mux)
   return mux->si_interval;
 }
 
+/**
+ * tsmux_resend_si:
+ * @mux: a #TsMux
+ *
+ * Resends the SI tables before the next stream packet.
+ *
+ */
+void
+tsmux_resend_si (TsMux * mux)
+{
+  g_return_if_fail (mux != NULL);
+
+  mux->last_si_ts = G_MININT64;
+}
+
 /**
  * tsmux_add_mpegts_si_section:
  * @mux: a #TsMux
@@ -427,6 +456,20 @@ tsmux_get_pmt_interval (TsMuxProgram * program)
   return program->pmt_interval;
 }
 
+/**
+ * tsmux_resend_pmt:
+ * @program: a #TsMuxProgram
+ *
+ * Resends the PMT before the next stream packet.
+ */
+void
+tsmux_resend_pmt (TsMuxProgram * program)
+{
+  g_return_if_fail (program != NULL);
+
+  program->last_pmt_ts = G_MININT64;
+}
+
 /**
  * tsmux_program_add_stream:
  * @program: a #TsMuxProgram
@@ -1104,8 +1147,7 @@ tsmux_write_stream_packet (TsMux * mux, TsMuxStream * stream)
 
   gst_buffer_unmap (buf, &map);
 
-  GST_DEBUG_OBJECT (mux, "Writing PES of size %d",
-      (int) gst_buffer_get_size (buf));
+  GST_DEBUG ("Writing PES of size %d", (int) gst_buffer_get_size (buf));
   res = tsmux_packet_out (mux, buf, cur_pcr);
 
   /* Reset all dynamic flags */
