@@ -23,15 +23,15 @@
  */
 /**
  * SECTION:element-stereo
+ * @title: stereo
  *
  * Create a wide stereo effect.
  *
- * <refsect2>
- * <title>Example pipelines</title>
+ * ## Example pipelines
  * |[
  * gst-launch-1.0 -v filesrc location=sine.ogg ! oggdemux ! vorbisdec ! audioconvert ! stereo ! audioconvert ! audioresample ! alsasink
  * ]| Play an Ogg/Vorbis file.
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -210,5 +210,5 @@ plugin_init (GstPlugin * plugin)
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
     stereo,
-    "Muck with the stereo signal, enhance it's 'stereo-ness'",
+    "Muck with the stereo signal, enhance its 'stereo-ness'",
     plugin_init, VERSION, GST_LICENSE, GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN)
