@@ -1,5 +1,5 @@
 /* GStreamer JPEG 2000 Parser
- * Copyright (C) <2016> Grok Image Compession Inc.
+ * Copyright (C) <2016-2017> Grok Image Compression Inc.
  *  @author Aaron Boxer <boxerab@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
@@ -25,6 +25,35 @@
 #include "gstjpeg2000parse.h"
 #include <gst/base/base.h>
 
+/* Not used at the moment
+static gboolean gst_jpeg2000_parse_is_cinema(guint16 rsiz)   {
+	return ((rsiz >= GST_JPEG2000_PARSE_PROFILE_CINEMA_2K) && (rsiz <= GST_JPEG2000_PARSE_PROFILE_CINEMA_S4K));
+}
+static gboolean gst_jpeg2000_parse_is_storage(guint16 rsiz)   {
+	return (rsiz == GST_JPEG2000_PARSE_PROFILE_CINEMA_LTS);
+}
+*/
+static gboolean
+gst_jpeg2000_parse_is_broadcast (guint16 rsiz)
+{
+  return ((rsiz >= GST_JPEG2000_PARSE_PROFILE_BC_SINGLE) &&
+      (rsiz <= ((GST_JPEG2000_PARSE_PROFILE_BC_MULTI_R) | (0x000b)))
+      && ((rsiz & (~GST_JPEG2000_PARSE_PROFILE_BC_MASK)) == 0));
+}
+
+static gboolean
+gst_jpeg2000_parse_is_imf (guint16 rsiz)
+{
+  return ((rsiz >= GST_JPEG2000_PARSE_PROFILE_IMF_2K)
+      && (rsiz <= ((GST_JPEG2000_PARSE_PROFILE_IMF_8K_R) | (0x009b))));
+}
+
+static gboolean
+gst_jpeg2000_parse_is_part_2 (guint16 rsiz)
+{
+  return (rsiz & GST_JPEG2000_PARSE_PROFILE_PART2);
+}
+
 
 
 static void
@@ -67,24 +96,19 @@ static GstStaticPadTemplate srctemplate =
         " width = (int)[1, MAX], height = (int)[1, MAX],"
         GST_JPEG2000_SAMPLING_LIST ","
         GST_JPEG2000_COLORSPACE_LIST ","
+        " profile = (int)[0, 49151],"
         " parsed = (boolean) true;"
         "image/x-j2c,"
         " width = (int)[1, MAX], height = (int)[1, MAX],"
         GST_JPEG2000_SAMPLING_LIST ","
-        GST_JPEG2000_COLORSPACE_LIST "," " parsed = (boolean) true")
+        GST_JPEG2000_COLORSPACE_LIST ","
+        " profile = (int)[0, 49151]," " parsed = (boolean) true")
     );
 
 static GstStaticPadTemplate sinktemplate =
     GST_STATIC_PAD_TEMPLATE ("sink", GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("image/x-jpc,"
-        GST_JPEG2000_SAMPLING_LIST ";"
-        "image/x-jpc, "
-        GST_JPEG2000_COLORSPACE_LIST ";"
-        "image/x-j2c,"
-        GST_JPEG2000_SAMPLING_LIST ";"
-        "image/x-j2c, " GST_JPEG2000_COLORSPACE_LIST)
-    );
+    GST_STATIC_CAPS ("image/jp2;image/x-jpc;image/x-j2c"));
 
 #define parent_class gst_jpeg2000_parse_parent_class
 G_DEFINE_TYPE (GstJPEG2000Parse, gst_jpeg2000_parse, GST_TYPE_BASE_PARSE);
@@ -177,6 +201,78 @@ gst_jpeg2000_parse_event (GstBaseParse * parse, GstEvent * event)
   return res;
 }
 
+static GstJPEG2000ParseFormats
+format_from_media_type (const GstStructure * structure)
+{
+  const char *media_type = gst_structure_get_name (structure);
+  if (!strcmp (media_type, "image/x-j2c"))
+    return GST_JPEG2000_PARSE_J2C;
+  if (!strcmp (media_type, "image/x-jpc"))
+    return GST_JPEG2000_PARSE_JPC;
+  if (!strcmp (media_type, "image/x-jp2"))
+    return GST_JPEG2000_PARSE_JP2;
+  return GST_JPEG2000_PARSE_NO_CODEC;
+}
+
+/* check downstream caps to configure media type */
+static void
+gst_jpeg2000_parse_negotiate (GstJPEG2000Parse * parse, GstCaps * in_caps)
+{
+  GstCaps *caps;
+  guint codec_format = GST_JPEG2000_PARSE_NO_CODEC;
+
+  g_return_if_fail ((in_caps == NULL) || gst_caps_is_fixed (in_caps));
+
+  caps = gst_pad_get_allowed_caps (GST_BASE_PARSE_SRC_PAD (parse));
+  GST_DEBUG_OBJECT (parse, "allowed caps: %" GST_PTR_FORMAT, caps);
+
+  /* concentrate on leading structure, since decodebin parser
+   * capsfilter always includes parser template caps */
+  if (caps) {
+    caps = gst_caps_truncate (caps);
+    GST_DEBUG_OBJECT (parse, "negotiating with caps: %" GST_PTR_FORMAT, caps);
+  }
+
+  if (in_caps && caps) {
+    if (gst_caps_can_intersect (in_caps, caps)) {
+      GST_DEBUG_OBJECT (parse, "downstream accepts upstream caps");
+      codec_format =
+          format_from_media_type (gst_caps_get_structure (in_caps, 0));
+      gst_caps_unref (caps);
+      caps = NULL;
+    }
+  }
+
+  if (caps && !gst_caps_is_empty (caps)) {
+    /* fixate to avoid ambiguity with lists when parsing */
+    caps = gst_caps_fixate (caps);
+    codec_format = format_from_media_type (gst_caps_get_structure (caps, 0));
+  }
+
+  GST_DEBUG_OBJECT (parse, "selected codec format %d", codec_format);
+
+  parse->codec_format = codec_format;
+
+  if (caps)
+    gst_caps_unref (caps);
+}
+
+static const char *
+media_type_from_codec_format (GstJPEG2000ParseFormats f)
+{
+  switch (f) {
+    case GST_JPEG2000_PARSE_J2C:
+      return "image/x-j2c";
+    case GST_JPEG2000_PARSE_JP2:
+      return "image/x-jp2";
+    case GST_JPEG2000_PARSE_JPC:
+      return "image/x-jpc";
+    default:
+      g_assert_not_reached ();
+      return "invalid/x-invalid";
+  }
+}
+
 static GstFlowReturn
 gst_jpeg2000_parse_handle_frame (GstBaseParse * parse,
     GstBaseParseFrame * frame, gint * skipsize)
@@ -188,13 +284,17 @@ gst_jpeg2000_parse_handle_frame (GstBaseParse * parse,
   guint eoc_offset = 0;
   GstCaps *current_caps = NULL;
   GstStructure *current_caps_struct = NULL;
-  const gchar *colorspace_string = NULL;
   GstJPEG2000Colorspace colorspace = GST_JPEG2000_COLORSPACE_NONE;
   guint x0, y0, x1, y1;
   guint width = 0, height = 0;
   guint8 dx[GST_JPEG2000_PARSE_MAX_SUPPORTED_COMPONENTS];
   guint8 dy[GST_JPEG2000_PARSE_MAX_SUPPORTED_COMPONENTS];
   guint16 numcomps;
+  guint16 capabilities = 0;
+  guint16 profile = 0;
+  gboolean validate_main_level = FALSE;
+  guint8 main_level = 0;
+  guint8 sub_level = 0;
   guint16 compno;
   GstJPEG2000Sampling parsed_sampling = GST_JPEG2000_SAMPLING_NONE;
   const gchar *sink_sampling_string = NULL;
@@ -205,23 +305,50 @@ gst_jpeg2000_parse_handle_frame (GstBaseParse * parse,
   guint num_prefix_bytes = 0;   /* number of bytes to skip before actual code stream */
   GstCaps *src_caps = NULL;
   guint frame_size = 0;
-  gboolean is_j2c = jpeg2000parse->codec_format == GST_JPEG2000_PARSE_J2C;
+  gboolean is_j2c;
+  gboolean parsed_j2c_4cc = FALSE;
 
   if (!gst_buffer_map (frame->buffer, &map, GST_MAP_READ)) {
     GST_ERROR_OBJECT (jpeg2000parse, "Unable to map buffer");
     return GST_FLOW_ERROR;
   }
-
   gst_byte_reader_init (&reader, map.data, map.size);
-  num_prefix_bytes = GST_JPEG2000_MARKER_SIZE;
 
-  if (is_j2c) {
-    num_prefix_bytes +=
-        GST_JPEG2000_JP2_SIZE_OF_BOX_LEN + GST_JPEG2000_JP2_SIZE_OF_BOX_ID;
-    /* check for "jp2c" */
+  /* try to get from caps */
+  if (jpeg2000parse->codec_format == GST_JPEG2000_PARSE_NO_CODEC)
+    gst_jpeg2000_parse_negotiate (jpeg2000parse, NULL);
+
+  /* if we can't get from caps, then try to parse */
+  if (jpeg2000parse->codec_format == GST_JPEG2000_PARSE_NO_CODEC) {
+    /* check for "jp2c" box */
+    /* both jp2 and j2c will be found with this scan, and both will be treated as j2c format */
     j2c_box_id_offset = gst_byte_reader_masked_scan_uint32 (&reader, 0xffffffff,
         GST_MAKE_FOURCC ('j', 'p', '2', 'c'), 0,
         gst_byte_reader_get_remaining (&reader));
+    parsed_j2c_4cc = TRUE;
+    is_j2c = j2c_box_id_offset != -1;
+    jpeg2000parse->codec_format =
+        is_j2c ? GST_JPEG2000_PARSE_J2C : GST_JPEG2000_PARSE_JPC;
+
+  } else {
+    /* for now, just treat JP2 as J2C */
+    if (jpeg2000parse->codec_format == GST_JPEG2000_PARSE_JP2) {
+      jpeg2000parse->codec_format = GST_JPEG2000_PARSE_J2C;
+    }
+    is_j2c = jpeg2000parse->codec_format == GST_JPEG2000_PARSE_J2C;
+  }
+
+  num_prefix_bytes = GST_JPEG2000_MARKER_SIZE;
+  if (is_j2c) {
+    num_prefix_bytes +=
+        GST_JPEG2000_JP2_SIZE_OF_BOX_LEN + GST_JPEG2000_JP2_SIZE_OF_BOX_ID;
+    /* check for "jp2c" (may have already parsed j2c_box_id_offset if caps are empty) */
+    if (!parsed_j2c_4cc) {
+      j2c_box_id_offset =
+          gst_byte_reader_masked_scan_uint32 (&reader, 0xffffffff,
+          GST_MAKE_FOURCC ('j', 'p', '2', 'c'), 0,
+          gst_byte_reader_get_remaining (&reader));
+    }
 
     if (j2c_box_id_offset == -1) {
       GST_ELEMENT_ERROR (jpeg2000parse, STREAM, DECODE, NULL,
@@ -279,10 +406,47 @@ gst_jpeg2000_parse_handle_frame (GstBaseParse * parse,
       goto beach;
   }
 
-  /* 2 to skip marker size, and another 2 to skip rsiz field */
-  if (!gst_byte_reader_skip (&reader, num_prefix_bytes + 2 + 2))
+  /* 2 to skip marker size */
+  if (!gst_byte_reader_skip (&reader, num_prefix_bytes + 2))
     goto beach;
 
+  if (!gst_byte_reader_get_uint16_be (&reader, &capabilities))
+    goto beach;
+
+  profile = capabilities & GST_JPEG2000_PARSE_PROFILE_MASK;
+  if (!gst_jpeg2000_parse_is_part_2 (capabilities)) {
+    if ((profile > GST_JPEG2000_PARSE_PROFILE_CINEMA_LTS)
+        && !gst_jpeg2000_parse_is_broadcast (profile)
+        && !gst_jpeg2000_parse_is_imf (profile)) {
+      GST_ELEMENT_ERROR (jpeg2000parse, STREAM, DECODE, NULL,
+          ("Unrecognized JPEG 2000 profile %d", profile));
+      ret = GST_FLOW_ERROR;
+      goto beach;
+    }
+    if (gst_jpeg2000_parse_is_broadcast (profile)) {
+      main_level = capabilities & 0xF;
+      validate_main_level = TRUE;
+    } else if (gst_jpeg2000_parse_is_imf (profile)) {
+      main_level = capabilities & 0xF;
+      validate_main_level = TRUE;
+      sub_level = (capabilities >> 4) & 0xF;
+      if (sub_level > 9) {
+        GST_ELEMENT_ERROR (jpeg2000parse, STREAM, DECODE, NULL,
+            ("Sub level %d is invalid", sub_level));
+        ret = GST_FLOW_ERROR;
+        goto beach;
+      }
+    }
+    if (validate_main_level && main_level > 11) {
+      GST_ELEMENT_ERROR (jpeg2000parse, STREAM, DECODE, NULL,
+          ("Main level %d is invalid", main_level));
+      ret = GST_FLOW_ERROR;
+      goto beach;
+
+    }
+  }
+
+
   if (!gst_byte_reader_get_uint32_be (&reader, &x1))
     goto beach;
 
@@ -317,7 +481,7 @@ gst_jpeg2000_parse_handle_frame (GstBaseParse * parse,
   if (!gst_byte_reader_get_uint16_be (&reader, &numcomps))
     goto beach;
 
-  if (numcomps == 2 || numcomps > GST_JPEG2000_PARSE_MAX_SUPPORTED_COMPONENTS) {
+  if (numcomps == 0 || numcomps > GST_JPEG2000_PARSE_MAX_SUPPORTED_COMPONENTS) {
     GST_ELEMENT_ERROR (jpeg2000parse, STREAM, DECODE, NULL,
         ("Unsupported number of components %d", numcomps));
     ret = GST_FLOW_NOT_NEGOTIATED;
@@ -325,28 +489,46 @@ gst_jpeg2000_parse_handle_frame (GstBaseParse * parse,
   }
 
   current_caps = gst_pad_get_current_caps (GST_BASE_PARSE_SINK_PAD (parse));
-  if (!current_caps) {
-    GST_ERROR_OBJECT (jpeg2000parse, "Unable to get current caps");
-    ret = GST_FLOW_NOT_NEGOTIATED;
-    goto beach;
-  }
+  if (current_caps) {
+    const gchar *colorspace_string = NULL;
+    current_caps_struct = gst_caps_get_structure (current_caps, 0);
+    if (!current_caps_struct) {
+      GST_ERROR_OBJECT (jpeg2000parse,
+          "Unable to get structure of current caps struct");
+      ret = GST_FLOW_NOT_NEGOTIATED;
+      goto beach;
+    }
 
-  current_caps_struct = gst_caps_get_structure (current_caps, 0);
-  if (!current_caps_struct) {
-    GST_ERROR_OBJECT (jpeg2000parse,
-        "Unable to get structure of current caps struct");
-    ret = GST_FLOW_NOT_NEGOTIATED;
-    goto beach;
-  }
+    colorspace_string = gst_structure_get_string
+        (current_caps_struct, "colorspace");
+    if (colorspace_string)
+      colorspace = gst_jpeg2000_colorspace_from_string (colorspace_string);
+    sink_sampling_string = gst_structure_get_string
+        (current_caps_struct, "sampling");
+    if (sink_sampling_string)
+      sink_sampling = gst_jpeg2000_sampling_from_string (sink_sampling_string);
 
-  colorspace_string = gst_structure_get_string
-      (current_caps_struct, "colorspace");
-  if (colorspace_string)
-    colorspace = gst_jpeg2000_colorspace_from_string (colorspace_string);
-  sink_sampling_string = gst_structure_get_string
-      (current_caps_struct, "sampling");
-  if (sink_sampling_string)
-    sink_sampling = gst_jpeg2000_sampling_from_string (sink_sampling_string);
+  } else {
+    /* guess color space based on number of components       */
+    if (numcomps == 0 || numcomps > 4) {
+      GST_ERROR_OBJECT (jpeg2000parse,
+          "Unable to guess color space from number of components %d", numcomps);
+      ret = GST_FLOW_NOT_NEGOTIATED;
+      goto beach;
+    }
+    colorspace =
+        (numcomps >=
+        3) ? GST_JPEG2000_COLORSPACE_RGB : GST_JPEG2000_COLORSPACE_GRAY;
+    if (numcomps == 4) {
+      GST_WARNING_OBJECT (jpeg2000parse, "No caps available: assuming RGBA");
+    } else if (numcomps == 3) {
+      GST_WARNING_OBJECT (jpeg2000parse, "No caps available: assuming RGB");
+    } else if (numcomps == 2) {
+      GST_WARNING_OBJECT (jpeg2000parse,
+          "No caps available: assuming grayscale with alpha");
+    }
+
+  }
 
   for (compno = 0; compno < numcomps; ++compno) {
 
@@ -460,6 +642,8 @@ gst_jpeg2000_parse_handle_frame (GstBaseParse * parse,
     }
   }
 
+  gst_jpeg2000_parse_negotiate (jpeg2000parse, current_caps);
+
   /* now we can set the source caps, if something has changed */
   source_sampling =
       sink_sampling !=
@@ -475,19 +659,65 @@ gst_jpeg2000_parse_handle_frame (GstBaseParse * parse,
     jpeg2000parse->colorspace = colorspace;
 
     src_caps =
-        gst_caps_new_simple (gst_structure_get_name (current_caps_struct),
-        "width", G_TYPE_INT, width, "height", G_TYPE_INT, height,
+        gst_caps_new_simple (media_type_from_codec_format
+        (jpeg2000parse->codec_format),
+        "width", G_TYPE_INT, width,
+        "height", G_TYPE_INT, height,
         "colorspace", G_TYPE_STRING,
         gst_jpeg2000_colorspace_to_string (colorspace), "sampling",
-        G_TYPE_STRING, gst_jpeg2000_sampling_to_string (source_sampling), NULL);
+        G_TYPE_STRING, gst_jpeg2000_sampling_to_string (source_sampling),
+        "profile", G_TYPE_UINT, profile, NULL);
 
-    if (gst_structure_get_fraction (current_caps_struct, "framerate", &fr_num,
-            &fr_denom)) {
-      gst_caps_set_simple (src_caps, "framerate", GST_TYPE_FRACTION, fr_num,
-          fr_denom, NULL);
-    } else {
-      GST_WARNING_OBJECT (jpeg2000parse, "No framerate set");
+    if (gst_jpeg2000_parse_is_broadcast (capabilities)
+        || gst_jpeg2000_parse_is_imf (capabilities)) {
+      gst_caps_set_simple (src_caps, "main-level", G_TYPE_UINT, main_level,
+          NULL);
+      if (gst_jpeg2000_parse_is_imf (capabilities)) {
+        gst_caps_set_simple (src_caps, "sub-level", G_TYPE_UINT, sub_level,
+            NULL);
+      }
     }
+
+    if (current_caps_struct) {
+      const gchar *caps_string = gst_structure_get_string
+          (current_caps_struct, "colorimetry");
+      if (caps_string) {
+        gst_caps_set_simple (src_caps, "colorimetry", G_TYPE_STRING,
+            caps_string, NULL);
+      }
+      caps_string = gst_structure_get_string
+          (current_caps_struct, "interlace-mode");
+      if (caps_string) {
+        gst_caps_set_simple (src_caps, "interlace-mode", G_TYPE_STRING,
+            caps_string, NULL);
+      }
+      caps_string = gst_structure_get_string
+          (current_caps_struct, "field-order");
+      if (caps_string) {
+        gst_caps_set_simple (src_caps, "field-order", G_TYPE_STRING,
+            caps_string, NULL);
+      }
+      caps_string = gst_structure_get_string
+          (current_caps_struct, "multiview-mode");
+      if (caps_string) {
+        gst_caps_set_simple (src_caps, "multiview-mode", G_TYPE_STRING,
+            caps_string, NULL);
+      }
+      caps_string = gst_structure_get_string
+          (current_caps_struct, "chroma-site");
+      if (caps_string) {
+        gst_caps_set_simple (src_caps, "chroma-site", G_TYPE_STRING,
+            caps_string, NULL);
+      }
+      if (gst_structure_get_fraction (current_caps_struct, "framerate", &fr_num,
+              &fr_denom)) {
+        gst_caps_set_simple (src_caps, "framerate", GST_TYPE_FRACTION, fr_num,
+            fr_denom, NULL);
+      } else {
+        GST_WARNING_OBJECT (jpeg2000parse, "No framerate set");
+      }
+    }
+
     if (!gst_pad_set_caps (GST_BASE_PARSE_SRC_PAD (parse), src_caps)) {
       GST_ERROR_OBJECT (jpeg2000parse, "Unable to set source caps");
       ret = GST_FLOW_NOT_NEGOTIATED;
@@ -520,6 +750,8 @@ gst_jpeg2000_parse_handle_frame (GstBaseParse * parse,
           frame_size, eoc_frame_size);
     }
     frame_size = eoc_frame_size;
+  } else {
+    goto beach;
   }
 
   /* clean up and finish frame */
