@@ -68,6 +68,7 @@ static void gst_avdtp_src_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 
 static GstCaps *gst_avdtp_src_getcaps (GstBaseSrc * bsrc, GstCaps * filter);
+static gboolean gst_avdtp_src_query (GstBaseSrc * bsrc, GstQuery * query);
 static gboolean gst_avdtp_src_start (GstBaseSrc * bsrc);
 static gboolean gst_avdtp_src_stop (GstBaseSrc * bsrc);
 static GstFlowReturn gst_avdtp_src_create (GstBaseSrc * bsrc, guint64 offset,
@@ -94,6 +95,7 @@ gst_avdtp_src_class_init (GstAvdtpSrcClass * klass)
   basesrc_class->unlock = GST_DEBUG_FUNCPTR (gst_avdtp_src_unlock);
   basesrc_class->unlock_stop = GST_DEBUG_FUNCPTR (gst_avdtp_src_unlock_stop);
   basesrc_class->get_caps = GST_DEBUG_FUNCPTR (gst_avdtp_src_getcaps);
+  basesrc_class->query = GST_DEBUG_FUNCPTR (gst_avdtp_src_query);
 
   g_object_class_install_property (gobject_class, PROP_TRANSPORT,
       g_param_spec_string ("transport",
@@ -108,8 +110,8 @@ gst_avdtp_src_class_init (GstAvdtpSrcClass * klass)
   GST_DEBUG_CATEGORY_INIT (avdtpsrc_debug, "avdtpsrc", 0,
       "Bluetooth AVDTP Source");
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_avdtp_src_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_avdtp_src_template);
 }
 
 static void
@@ -117,6 +119,8 @@ gst_avdtp_src_init (GstAvdtpSrc * avdtpsrc)
 {
   avdtpsrc->poll = gst_poll_new (TRUE);
 
+  avdtpsrc->duration = GST_CLOCK_TIME_NONE;
+
   gst_base_src_set_format (GST_BASE_SRC (avdtpsrc), GST_FORMAT_TIME);
   gst_base_src_set_live (GST_BASE_SRC (avdtpsrc), TRUE);
   gst_base_src_set_do_timestamp (GST_BASE_SRC (avdtpsrc), TRUE);
@@ -169,6 +173,35 @@ gst_avdtp_src_set_property (GObject * object, guint prop_id,
   }
 }
 
+static gboolean
+gst_avdtp_src_query (GstBaseSrc * bsrc, GstQuery * query)
+{
+  GstAvdtpSrc *avdtpsrc = GST_AVDTP_SRC (bsrc);
+  gboolean ret = FALSE;
+
+  switch (GST_QUERY_TYPE (query)) {
+    case GST_QUERY_DURATION:{
+      GstFormat format;
+
+      if (avdtpsrc->duration != GST_CLOCK_TIME_NONE) {
+        gst_query_parse_duration (query, &format, NULL);
+
+        if (format == GST_FORMAT_TIME) {
+          gst_query_set_duration (query, format, (gint64) avdtpsrc->duration);
+          ret = TRUE;
+        }
+      }
+
+      break;
+    }
+
+    default:
+      ret = GST_BASE_SRC_CLASS (parent_class)->query (bsrc, query);
+  }
+
+  return ret;
+}
+
 static GstCaps *
 gst_avdtp_src_getcaps (GstBaseSrc * bsrc, GstCaps * filter)
 {
@@ -199,6 +232,7 @@ gst_avdtp_src_getcaps (GstBaseSrc * bsrc, GstCaps * filter)
       value = gst_structure_get_value (structure, "mpegversion");
       if (!value || !G_VALUE_HOLDS_INT (value)) {
         GST_ERROR_OBJECT (avdtpsrc, "Failed to get mpegversion");
+        gst_caps_unref (caps);
         return NULL;
       }
       gst_caps_set_simple (caps, "mpegversion", G_TYPE_INT,
@@ -207,6 +241,7 @@ gst_avdtp_src_getcaps (GstBaseSrc * bsrc, GstCaps * filter)
       value = gst_structure_get_value (structure, "channels");
       if (!value || !G_VALUE_HOLDS_INT (value)) {
         GST_ERROR_OBJECT (avdtpsrc, "Failed to get channels");
+        gst_caps_unref (caps);
         return NULL;
       }
       gst_caps_set_simple (caps, "channels", G_TYPE_INT,
@@ -215,6 +250,7 @@ gst_avdtp_src_getcaps (GstBaseSrc * bsrc, GstCaps * filter)
       value = gst_structure_get_value (structure, "base-profile");
       if (!value || !G_VALUE_HOLDS_STRING (value)) {
         GST_ERROR_OBJECT (avdtpsrc, "Failed to get base-profile");
+        gst_caps_unref (caps);
         return NULL;
       }
       gst_caps_set_simple (caps, "base-profile", G_TYPE_STRING,
@@ -228,6 +264,7 @@ gst_avdtp_src_getcaps (GstBaseSrc * bsrc, GstCaps * filter)
     value = gst_structure_get_value (structure, "rate");
     if (!value || !G_VALUE_HOLDS_INT (value)) {
       GST_ERROR_OBJECT (avdtpsrc, "Failed to get sample rate");
+      gst_caps_unref (caps);
       return NULL;
     }
     rate = g_value_get_int (value);
@@ -247,6 +284,54 @@ gst_avdtp_src_getcaps (GstBaseSrc * bsrc, GstCaps * filter)
   return ret;
 }
 
+static void
+avrcp_metadata_cb (GstAvrcpConnection * avrcp, GstTagList * taglist,
+    gpointer user_data)
+{
+  GstAvdtpSrc *src = GST_AVDTP_SRC (user_data);
+  guint64 duration;
+
+  if (gst_tag_list_get_uint64 (taglist, GST_TAG_DURATION, &duration)) {
+    src->duration = duration;
+    gst_element_post_message (GST_ELEMENT (src),
+        gst_message_new_duration_changed (GST_OBJECT (src)));
+  }
+
+  gst_pad_push_event (GST_BASE_SRC_PAD (src),
+      gst_event_new_tag (gst_tag_list_copy (taglist)));
+  gst_element_post_message (GST_ELEMENT (src),
+      gst_message_new_tag (GST_OBJECT (src), taglist));
+}
+
+static void
+gst_avdtp_src_start_avrcp (GstAvdtpSrc * src)
+{
+  gchar *path, **strv;
+  int i;
+
+  /* Strip out the /fdX in /org/bluez/dev_.../fdX */
+  strv = g_strsplit (src->conn.transport, "/", -1);
+
+  for (i = 0; strv[i]; i++);
+  g_return_if_fail (i > 0);
+
+  g_free (strv[i - 1]);
+  strv[i - 1] = NULL;
+
+  path = g_strjoinv ("/", strv);
+  g_strfreev (strv);
+
+  src->avrcp = gst_avrcp_connection_new (path, avrcp_metadata_cb, src, NULL);
+
+  g_free (path);
+}
+
+static void
+gst_avdtp_src_stop_avrcp (GstAvdtpSrc * src)
+{
+  gst_avrcp_connection_free (src->avrcp);
+}
+
 static gboolean
 gst_avdtp_src_start (GstBaseSrc * bsrc)
 {
@@ -291,6 +376,8 @@ gst_avdtp_src_start (GstBaseSrc * bsrc)
 
   g_atomic_int_set (&avdtpsrc->unlocked, FALSE);
 
+  gst_avdtp_src_start_avrcp (avdtpsrc);
+
   return TRUE;
 
 fail:
@@ -306,6 +393,7 @@ gst_avdtp_src_stop (GstBaseSrc * bsrc)
   gst_poll_remove_fd (avdtpsrc->poll, &avdtpsrc->pfd);
   gst_poll_set_flushing (avdtpsrc->poll, TRUE);
 
+  gst_avdtp_src_stop_avrcp (avdtpsrc);
   gst_avdtp_connection_release (&avdtpsrc->conn);
 
   if (avdtpsrc->dev_caps) {
