@@ -777,12 +777,18 @@ public:
       VideoInputFormatChanged (BMDVideoInputFormatChangedEvents,
       IDeckLinkDisplayMode * mode, BMDDetectedVideoInputFormatFlags formatFlags)
   {
-    BMDPixelFormat pixelFormat = bmdFormat8BitYUV;
+    BMDPixelFormat pixelFormat;
 
     GST_INFO ("Video input format changed");
 
-    if (formatFlags & bmdDetectedVideoInputRGB444)
+    if ((formatFlags & bmdDetectedVideoInputRGB444)
+        && m_input->format == bmdFormat8BitYUV) {
+      /* user-set format was auto or 8BitYUV, change to RGB */
       pixelFormat = bmdFormat8BitARGB;
+    } else {
+      /* use the user-set format, defaulting to 8BitYUV */
+      pixelFormat = m_input->format;
+    }
 
     g_mutex_lock (&m_input->lock);
     m_input->input->PauseStreams ();
@@ -1065,7 +1071,7 @@ public:
   }
 };
 
-#ifdef _MSC_VER
+#ifdef G_OS_WIN32
 /* FIXME: We currently never deinit this */
 
 static GMutex com_init_lock;
@@ -1114,7 +1120,7 @@ gst_decklink_com_thread (gpointer data)
 
   return NULL;
 }
-#endif /* _MSC_VER */
+#endif /* G_OS_WIN32 */
 
 static GOnce devices_once = G_ONCE_INIT;
 static GPtrArray *devices;      /* array of Device */
@@ -1127,18 +1133,18 @@ init_devices (gpointer data)
   HRESULT ret;
   int i;
 
-#ifdef _MSC_VER
+#ifdef G_OS_WIN32
   // Start COM thread for Windows
 
   g_mutex_lock (&com_init_lock);
 
   /* create the COM initialization thread */
-  g_thread_create ((GThreadFunc) gst_decklink_com_thread, NULL, FALSE, NULL);
+  g_thread_new ("COM init thread", (GThreadFunc) gst_decklink_com_thread, NULL);
 
   /* wait until the COM thread signals that COM has been initialized */
   g_cond_wait (&com_init_cond, &com_init_lock);
   g_mutex_unlock (&com_init_lock);
-#endif /* _MSC_VER */
+#endif /* G_OS_WIN32 */
 
   iterator = CreateDeckLinkIteratorInstance ();
   if (iterator == NULL) {
