@@ -31,10 +31,6 @@
 
 #include "windows.h"
 
-#ifdef _WIN64
-#define GWL_WNDPROC GWLP_WNDPROC
-#endif
-
 #define WM_GRAPH_NOTIFY WM_APP + 1 /* Private message */
 
 GST_DEBUG_CATEGORY (dshowvideosink_debug);
@@ -129,7 +125,7 @@ gst_dshowvideosink_set_window_handle (GstVideoOverlay * overlay, guintptr window
       sink->window_closed = FALSE;
     } else {
       /* Return control of application window */
-      SetWindowLongPtr (previous_window, GWL_WNDPROC, (LONG)sink->prevWndProc);
+      SetWindowLongPtr (previous_window, GWLP_WNDPROC, (LONG_PTR)sink->prevWndProc);
       SetWindowPos (previous_window, 0, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER | SWP_FRAMECHANGED);
     }
 
@@ -210,8 +206,7 @@ gst_dshowvideosink_class_init (GstDshowVideoSinkClass * klass)
       "Pioneers of the Inevitable <songbird@songbirdnest.com>, " \
       "FLUENDO S.A. <support@fluendo.com>");
 
-  gst_element_class_add_pad_template (e_class,
-      gst_static_pad_template_get (&sink_template));
+  gst_element_class_add_static_pad_template (e_class, &sink_template);
 
   e_class->change_state = GST_DEBUG_FUNCPTR (gst_dshowvideosink_change_state);
 
@@ -289,8 +284,7 @@ gst_dshowvideosink_finalize (GObject * gobject)
 {
   GstDshowVideoSink *sink = GST_DSHOWVIDEOSINK (gobject);
 
-  if (sink->preferredrenderer)
-    g_free (sink->preferredrenderer);
+  g_free (sink->preferredrenderer);
 
   /* signal the COM thread that it sould uninitialize COM */
   if (sink->comInitialized) {
@@ -319,9 +313,7 @@ gst_dshowvideosink_set_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case PROP_RENDERER:
-      if (sink->preferredrenderer)
-        g_free (sink->preferredrenderer);
-
+      g_free (sink->preferredrenderer);
       sink->preferredrenderer = g_value_dup_string (value);
       break;
     case PROP_KEEP_ASPECT_RATIO:
@@ -678,7 +670,7 @@ gst_dshowvideosink_window_thread (GstDshowVideoSink * sink)
 
   sink->is_new_window = TRUE;
 
-  SetWindowLongPtr (video_window, GWLP_USERDATA, (LONG)sink);
+  SetWindowLongPtr (video_window, GWLP_USERDATA, (LONG_PTR)sink);
 
   sink->window_id = video_window;
 
@@ -747,7 +739,7 @@ failed:
 
 static void gst_dshowvideosink_set_window_for_renderer (GstDshowVideoSink *sink)
 {
-  WNDPROC prevWndProc = (WNDPROC)GetWindowLong (sink->window_id, GWL_WNDPROC);
+  WNDPROC prevWndProc = (WNDPROC)GetWindowLongPtr (sink->window_id, GWLP_WNDPROC);
   if (prevWndProc == WndProcHook) {
     /* The WndProc already points to our hook. Something has gone wrong
      * somewhere else and this safety net prevents an infinite recursion */
@@ -755,7 +747,7 @@ static void gst_dshowvideosink_set_window_for_renderer (GstDshowVideoSink *sink)
   }
 
   /* Application has requested a specific window ID */
-  sink->prevWndProc = (WNDPROC) SetWindowLong (sink->window_id, GWL_WNDPROC, (LONG)WndProcHook);
+  sink->prevWndProc = (WNDPROC) SetWindowLongPtr (sink->window_id, GWLP_WNDPROC, (LONG_PTR)WndProcHook);
   GST_DEBUG_OBJECT (sink, "Set wndproc to %p from %p", WndProcHook, sink->prevWndProc);
   SetProp (sink->window_id, (LPCSTR)"GstDShowVideoSink", sink);
   /* This causes the new WNDPROC to become active */
@@ -948,7 +940,7 @@ gst_dshowvideosink_stop_graph (GstDshowVideoSink *sink)
 
   if (sink->window_id) {
     /* Return control of application window */
-    SetWindowLong (sink->window_id, GWL_WNDPROC, (LONG)sink->prevWndProc);
+    SetWindowLongPtr (sink->window_id, GWLP_WNDPROC, (LONG_PTR)sink->prevWndProc);
     RemoveProp (sink->window_id, (LPCSTR)"GstDShowVideoSink");
     SetWindowPos (sink->window_id, 0, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER | SWP_FRAMECHANGED);
     sink->prevWndProc = NULL;
