@@ -175,6 +175,7 @@ ensure_bitrate_control (GstMsdkEnc * thiz)
     case MFX_RATECONTROL_AVBR:
       mfx->Accuracy = thiz->accuracy;
       mfx->Convergence = thiz->convergence;
+      break;
 
     case MFX_RATECONTROL_VBR:
       option2->MaxFrameSize = thiz->max_frame_size * 1000;
@@ -194,8 +195,8 @@ ensure_bitrate_control (GstMsdkEnc * thiz)
   }
 }
 
-static void
-ensure_extended_coding_options (GstMsdkEnc * thiz)
+void
+gst_msdkenc_ensure_extended_coding_options (GstMsdkEnc * thiz)
 {
   mfxExtCodingOption2 *option2 = &thiz->option2;
   mfxExtCodingOption3 *option3 = &thiz->option3;
@@ -339,9 +340,14 @@ gst_msdkenc_init_encoder (GstMsdkEnc * thiz)
 
     status = MFXVideoVPP_GetVideoParam (session, &thiz->vpp_param);
     if (status < MFX_ERR_NONE) {
+      mfxStatus status1;
       GST_ERROR_OBJECT (thiz, "Get VPP Parameters failed (%s)",
           msdk_status_to_string (status));
-      MFXVideoVPP_Close (session);
+      status1 = MFXVideoVPP_Close (session);
+      if (status1 != MFX_ERR_NONE && status1 != MFX_ERR_NOT_INITIALIZED)
+        GST_WARNING_OBJECT (thiz, "VPP close failed (%s)",
+            msdk_status_to_string (status1));
+
       goto no_vpp;
     } else if (status > MFX_ERR_NONE) {
       GST_WARNING_OBJECT (thiz, "Get VPP Parameters returned: %s",
@@ -380,9 +386,6 @@ gst_msdkenc_init_encoder (GstMsdkEnc * thiz)
   /* ensure bitrate control parameters */
   ensure_bitrate_control (thiz);
 
-  /* Enable ExtCodingOption2 */
-  ensure_extended_coding_options (thiz);
-
   /* allow subclass configure further */
   if (klass->configure) {
     if (!klass->configure (thiz))
@@ -663,9 +666,14 @@ gst_msdkenc_finish_frame (GstMsdkEnc * thiz, MsdkEncTask * task,
     return GST_FLOW_ERROR;
   }
 
-  /* Wait for encoding operation to complete */
-  MFXVideoCORE_SyncOperation (gst_msdk_context_get_session (thiz->context),
-      task->sync_point, 10000);
+  /* Wait for encoding operation to complete, the magic number 300000 below
+   * is used in MSDK samples
+   * #define MSDK_ENC_WAIT_INTERVAL 300000
+   */
+  if (MFXVideoCORE_SyncOperation (gst_msdk_context_get_session (thiz->context),
+          task->sync_point, 300000) != MFX_ERR_NONE)
+    GST_WARNING_OBJECT (thiz, "failed to do sync operation");
+
   if (!discard && task->output_bitstream.DataLength) {
     GstBuffer *out_buf = NULL;
     guint8 *data =
@@ -805,6 +813,7 @@ gst_msdkenc_flush_frames (GstMsdkEnc * thiz, gboolean discard)
     if (status != MFX_ERR_NONE && status != MFX_ERR_MORE_DATA) {
       GST_ELEMENT_ERROR (thiz, STREAM, ENCODE, ("Encode frame failed."),
           ("MSDK encode error (%s)", msdk_status_to_string (status)));
+      break;
     }
 
     if (task->sync_point) {
@@ -875,7 +884,7 @@ gst_msdkenc_create_buffer_pool (GstMsdkEnc * thiz, GstCaps * caps,
 
   if (!gst_video_info_from_caps (&info, caps)) {
     GST_INFO_OBJECT (thiz, "failed to get video info");
-    return FALSE;
+    return NULL;
   }
 
   gst_msdk_set_video_alignment (&info, &align);
@@ -914,17 +923,20 @@ gst_msdkenc_create_buffer_pool (GstMsdkEnc * thiz, GstCaps * caps,
 error_no_pool:
   {
     GST_INFO_OBJECT (thiz, "failed to create bufferpool");
-    return FALSE;
+    return NULL;
   }
 error_no_allocator:
   {
     GST_INFO_OBJECT (thiz, "failed to create allocator");
-    return FALSE;
+    gst_object_unref (pool);
+    return NULL;
   }
 error_pool_config:
   {
     GST_INFO_OBJECT (thiz, "failed to set config");
-    return FALSE;
+    gst_object_unref (pool);
+    gst_object_unref (allocator);
+    return NULL;
   }
 }
 
