@@ -193,6 +193,10 @@ struct gl_input_resource
   gsize cuda_num_bytes;
   NV_ENC_REGISTER_RESOURCE nv_resource;
   NV_ENC_MAP_INPUT_RESOURCE nv_mapped_resource;
+
+  /* whether nv_mapped_resource was mapped via NvEncMapInputResource()
+   * and therefore should unmap via NvEncUnmapInputResource or not */
+  gboolean mapped;
 };
 #endif
 
@@ -224,7 +228,8 @@ static void gst_nv_base_enc_get_property (GObject * object, guint prop_id,
 static void gst_nv_base_enc_finalize (GObject * obj);
 static GstCaps *gst_nv_base_enc_getcaps (GstVideoEncoder * enc,
     GstCaps * filter);
-static gboolean gst_nv_base_enc_stop_bitstream_thread (GstNvBaseEnc * nvenc);
+static gboolean gst_nv_base_enc_stop_bitstream_thread (GstNvBaseEnc * nvenc,
+    gboolean force);
 
 static void
 gst_nv_base_enc_class_init (GstNvBaseEncClass * klass)
@@ -307,9 +312,11 @@ _get_supported_input_formats (GstNvBaseEnc * nvenc)
   guint64 format_mask = 0;
   uint32_t i, num = 0;
   NV_ENC_BUFFER_FORMAT formats[64];
-  GValue list = G_VALUE_INIT;
   GValue val = G_VALUE_INIT;
 
+  if (nvenc->input_formats)
+    return TRUE;
+
   NvEncGetInputFormats (nvenc->encoder, nvenc_class->codec_id, formats,
       G_N_ELEMENTS (formats), &num);
 
@@ -376,30 +383,30 @@ _get_supported_input_formats (GstNvBaseEnc * nvenc)
   if (format_mask == 0)
     return FALSE;
 
+  GST_OBJECT_LOCK (nvenc);
+  nvenc->input_formats = g_new0 (GValue, 1);
+
   /* process a second time so we can add formats in the order we want */
-  g_value_init (&list, GST_TYPE_LIST);
+  g_value_init (nvenc->input_formats, GST_TYPE_LIST);
   g_value_init (&val, G_TYPE_STRING);
   if ((format_mask & (1 << GST_VIDEO_FORMAT_NV12))) {
     g_value_set_static_string (&val, "NV12");
-    gst_value_list_append_value (&list, &val);
+    gst_value_list_append_value (nvenc->input_formats, &val);
   }
   if ((format_mask & (1 << GST_VIDEO_FORMAT_YV12))) {
     g_value_set_static_string (&val, "YV12");
-    gst_value_list_append_value (&list, &val);
+    gst_value_list_append_value (nvenc->input_formats, &val);
   }
   if ((format_mask & (1 << GST_VIDEO_FORMAT_I420))) {
     g_value_set_static_string (&val, "I420");
-    gst_value_list_append_value (&list, &val);
+    gst_value_list_append_value (nvenc->input_formats, &val);
   }
   if ((format_mask & (1 << GST_VIDEO_FORMAT_Y444))) {
     g_value_set_static_string (&val, "Y444");
-    gst_value_list_append_value (&list, &val);
+    gst_value_list_append_value (nvenc->input_formats, &val);
   }
   g_value_unset (&val);
 
-  GST_OBJECT_LOCK (nvenc);
-  g_free (nvenc->input_formats);
-  nvenc->input_formats = g_memdup (&list, sizeof (GValue));
   GST_OBJECT_UNLOCK (nvenc);
 
   return TRUE;
@@ -520,10 +527,15 @@ gst_nv_base_enc_stop (GstVideoEncoder * enc)
 {
   GstNvBaseEnc *nvenc = GST_NV_BASE_ENC (enc);
 
-  gst_nv_base_enc_stop_bitstream_thread (nvenc);
+  gst_nv_base_enc_stop_bitstream_thread (nvenc, TRUE);
 
   gst_nv_base_enc_free_buffers (nvenc);
 
+  if (nvenc->input_state) {
+    gst_video_codec_state_unref (nvenc->input_state);
+    nvenc->input_state = NULL;
+  }
+
   if (nvenc->bitstream_pool) {
     g_async_queue_unref (nvenc->bitstream_pool);
     nvenc->bitstream_pool = NULL;
@@ -573,7 +585,7 @@ _get_interlace_modes (GstNvBaseEnc * nvenc)
     g_value_set_static_string (&val, "interleaved");
     gst_value_list_append_value (list, &val);
     g_value_set_static_string (&val, "mixed");
-    gst_value_list_append_value (list, &val);
+    gst_value_list_append_and_take_value (list, &val);
   }
   /* TODO: figure out what nvenc frame based interlacing means in gst terms */
 
@@ -598,6 +610,7 @@ gst_nv_base_enc_getcaps (GstVideoEncoder * enc, GstCaps * filter)
 
     val = _get_interlace_modes (nvenc);
     gst_caps_set_value (supported_incaps, "interlace-mode", val);
+    g_value_unset (val);
     g_free (val);
 
     GST_LOG_OBJECT (enc, "codec input caps %" GST_PTR_FORMAT, supported_incaps);
@@ -639,6 +652,8 @@ gst_nv_base_enc_close (GstVideoEncoder * enc)
   }
 
   GST_OBJECT_LOCK (nvenc);
+  if (nvenc->input_formats)
+    g_value_unset (nvenc->input_formats);
   g_free (nvenc->input_formats);
   nvenc->input_formats = NULL;
   GST_OBJECT_UNLOCK (nvenc);
@@ -840,6 +855,8 @@ gst_nv_base_enc_bitstream_thread (gpointer user_data)
         nv_ret =
             NvEncUnmapInputResource (nvenc->encoder,
             in_gl_resource->nv_mapped_resource.mappedResource);
+        in_gl_resource->mapped = FALSE;
+
         if (nv_ret != NV_ENC_SUCCESS) {
           GST_ERROR_OBJECT (nvenc, "Failed to unmap input resource %p, ret %d",
               in_gl_resource, nv_ret);
@@ -892,29 +909,37 @@ gst_nv_base_enc_start_bitstream_thread (GstNvBaseEnc * nvenc)
 }
 
 static gboolean
-gst_nv_base_enc_stop_bitstream_thread (GstNvBaseEnc * nvenc)
+gst_nv_base_enc_stop_bitstream_thread (GstNvBaseEnc * nvenc, gboolean force)
 {
   gpointer out_buf;
 
   if (nvenc->bitstream_thread == NULL)
     return TRUE;
 
-  /* FIXME */
-  GST_FIXME_OBJECT (nvenc, "stop bitstream reading thread properly");
-  g_async_queue_lock (nvenc->bitstream_queue);
-  g_async_queue_lock (nvenc->bitstream_pool);
-  while ((out_buf = g_async_queue_try_pop_unlocked (nvenc->bitstream_queue))) {
-    GST_INFO_OBJECT (nvenc, "stole bitstream buffer %p from queue", out_buf);
-    g_async_queue_push_unlocked (nvenc->bitstream_pool, out_buf);
+  if (force) {
+    g_async_queue_lock (nvenc->bitstream_queue);
+    g_async_queue_lock (nvenc->bitstream_pool);
+    while ((out_buf = g_async_queue_try_pop_unlocked (nvenc->bitstream_queue))) {
+      GST_INFO_OBJECT (nvenc, "stole bitstream buffer %p from queue", out_buf);
+      g_async_queue_push_unlocked (nvenc->bitstream_pool, out_buf);
+    }
+    g_async_queue_push_unlocked (nvenc->bitstream_queue, SHUTDOWN_COOKIE);
+    g_async_queue_unlock (nvenc->bitstream_pool);
+    g_async_queue_unlock (nvenc->bitstream_queue);
+  } else {
+    /* wait for encoder to drain the remaining buffers */
+    g_async_queue_push (nvenc->bitstream_queue, SHUTDOWN_COOKIE);
+  }
+
+  if (!force) {
+    /* temporary unlock during finish, so other thread can find and push frame */
+    GST_VIDEO_ENCODER_STREAM_UNLOCK (nvenc);
   }
-  g_async_queue_push_unlocked (nvenc->bitstream_queue, SHUTDOWN_COOKIE);
-  g_async_queue_unlock (nvenc->bitstream_pool);
-  g_async_queue_unlock (nvenc->bitstream_queue);
 
-  /* temporary unlock, so other thread can find and push frame */
-  GST_VIDEO_ENCODER_STREAM_UNLOCK (nvenc);
   g_thread_join (nvenc->bitstream_thread);
-  GST_VIDEO_ENCODER_STREAM_LOCK (nvenc);
+
+  if (!force)
+    GST_VIDEO_ENCODER_STREAM_LOCK (nvenc);
 
   nvenc->bitstream_thread = NULL;
   return TRUE;
@@ -966,6 +991,20 @@ gst_nv_base_enc_free_buffers (GstNvBaseEnc * nvenc)
       struct gl_input_resource *in_gl_resource = nvenc->input_bufs[i];
 
       cuCtxPushCurrent (nvenc->cuda_ctx);
+
+      if (in_gl_resource->mapped) {
+        GST_LOG_OBJECT (nvenc, "Unmap resource %p", in_gl_resource);
+
+        nv_ret =
+            NvEncUnmapInputResource (nvenc->encoder,
+            in_gl_resource->nv_mapped_resource.mappedResource);
+
+        if (nv_ret != NV_ENC_SUCCESS) {
+          GST_ERROR_OBJECT (nvenc, "Failed to unmap input resource %p, ret %d",
+              in_gl_resource, nv_ret);
+        }
+      }
+
       nv_ret =
           NvEncUnregisterResource (nvenc->encoder,
           in_gl_resource->nv_resource.registeredResource);
@@ -973,6 +1012,12 @@ gst_nv_base_enc_free_buffers (GstNvBaseEnc * nvenc)
         GST_ERROR_OBJECT (nvenc, "Failed to unregister resource %p, ret %d",
             in_gl_resource, nv_ret);
 
+      nv_ret = cuMemFree ((CUdeviceptr) in_gl_resource->cuda_pointer);
+      if (nv_ret != NV_ENC_SUCCESS) {
+        GST_ERROR_OBJECT (nvenc, "Failed to free CUDA device memory, ret %d",
+            nv_ret);
+      }
+
       g_free (in_gl_resource);
       cuCtxPopCurrent (NULL);
     } else
@@ -1718,6 +1763,8 @@ gst_nv_base_enc_handle_frame (GstVideoEncoder * enc, GstVideoCodecFrame * frame)
       goto error;
     }
 
+    in_gl_resource->mapped = TRUE;
+
     out_buf = g_async_queue_try_pop (nvenc->bitstream_pool);
     if (out_buf == NULL) {
       GST_DEBUG_OBJECT (nvenc, "wait for output buf to become available again");
@@ -1900,12 +1947,10 @@ gst_nv_base_enc_finish (GstVideoEncoder * enc)
 {
   GstNvBaseEnc *nvenc = GST_NV_BASE_ENC (enc);
 
-  GST_FIXME_OBJECT (enc, "implement finish");
+  if (!gst_nv_base_enc_drain_encoder (nvenc))
+    return GST_FLOW_ERROR;
 
-  gst_nv_base_enc_drain_encoder (nvenc);
-
-  /* wait for encoder to output the remaining buffers */
-  gst_nv_base_enc_stop_bitstream_thread (nvenc);
+  gst_nv_base_enc_stop_bitstream_thread (nvenc, FALSE);
 
   return GST_FLOW_OK;
 }
