@@ -20,17 +20,18 @@
  */
 /**
  * SECTION:element-shmsrc
+ * @title: shmsrc
  *
  * Receive data from the shared memory sink.
  *
- * <refsect2>
- * <title>Example launch lines</title>
+ * ## Example launch lines
  * |[
  * gst-launch-1.0 shmsrc socket-path=/tmp/blah ! \
- * "video/x-yuv, format=YUY2, color-matrix=sdtv, \
- * chroma-site=mpeg2, width=(int)320, height=(int)240, framerate=(fraction)30/1" ! autovideosink
+ * "video/x-raw, format=YUY2, color-matrix=sdtv, \
+ * chroma-site=mpeg2, width=(int)320, height=(int)240, framerate=(fraction)30/1" \
+ * ! queue ! videoconvert ! autovideosink
  * ]| Render video from shm buffers.
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -90,7 +91,6 @@ static gboolean gst_shm_src_unlock_stop (GstBaseSrc * bsrc);
 static GstStateChangeReturn gst_shm_src_change_state (GstElement * element,
     GstStateChange transition);
 
-static void gst_shm_pipe_inc (GstShmPipe * pipe);
 static void gst_shm_pipe_dec (GstShmPipe * pipe);
 
 // static guint gst_shm_src_signals[LAST_SIGNAL] = { 0 };
@@ -254,6 +254,7 @@ gst_shm_src_start_reading (GstShmSrc * self)
 
   self->pipe = gstpipe;
 
+  self->unlocked = FALSE;
   gst_poll_set_flushing (self->poll, FALSE);
 
   gst_poll_fd_init (&self->pollfd);
@@ -267,16 +268,18 @@ gst_shm_src_start_reading (GstShmSrc * self)
 static void
 gst_shm_src_stop_reading (GstShmSrc * self)
 {
+  GstShmPipe *pipe;
+
   GST_DEBUG_OBJECT (self, "Stopping %p", self);
 
-  if (self->pipe) {
-    gst_shm_pipe_dec (self->pipe);
-    self->pipe = NULL;
+  GST_OBJECT_LOCK (self);
+  pipe = self->pipe;
+  self->pipe = NULL;
+  GST_OBJECT_UNLOCK (self);
 
-    gst_poll_remove_fd (self->poll, &self->pollfd);
+  if (pipe) {
+    gst_shm_pipe_dec (pipe);
   }
-
-  gst_poll_fd_init (&self->pollfd);
   gst_poll_set_flushing (self->poll, TRUE);
 }
 
@@ -321,44 +324,57 @@ static GstFlowReturn
 gst_shm_src_create (GstPushSrc * psrc, GstBuffer ** outbuf)
 {
   GstShmSrc *self = GST_SHM_SRC (psrc);
+  GstShmPipe *pipe;
   gchar *buf = NULL;
   int rv = 0;
   struct GstShmBuffer *gsb;
 
+  GST_DEBUG_OBJECT (self, "Stopping %p", self);
+
+  GST_OBJECT_LOCK (self);
+  pipe = self->pipe;
+  if (!pipe) {
+    GST_OBJECT_UNLOCK (self);
+    return GST_FLOW_FLUSHING;
+  } else {
+    pipe->use_count++;
+  }
+  GST_OBJECT_UNLOCK (self);
+
   do {
     if (gst_poll_wait (self->poll, GST_CLOCK_TIME_NONE) < 0) {
       if (errno == EBUSY)
-        return GST_FLOW_FLUSHING;
+        goto flushing;
       GST_ELEMENT_ERROR (self, RESOURCE, READ, ("Failed to read from shmsrc"),
           ("Poll failed on fd: %s", strerror (errno)));
-      return GST_FLOW_ERROR;
+      goto error;
     }
 
     if (self->unlocked)
-      return GST_FLOW_FLUSHING;
+      goto flushing;
 
     if (gst_poll_fd_has_closed (self->poll, &self->pollfd)) {
       GST_ELEMENT_ERROR (self, RESOURCE, READ, ("Failed to read from shmsrc"),
           ("Control socket has closed"));
-      return GST_FLOW_ERROR;
+      goto error;
     }
 
     if (gst_poll_fd_has_error (self->poll, &self->pollfd)) {
       GST_ELEMENT_ERROR (self, RESOURCE, READ, ("Failed to read from shmsrc"),
           ("Control socket has error"));
-      return GST_FLOW_ERROR;
+      goto error;
     }
 
     if (gst_poll_fd_can_read (self->poll, &self->pollfd)) {
       buf = NULL;
       GST_LOG_OBJECT (self, "Reading from pipe");
       GST_OBJECT_LOCK (self);
-      rv = sp_client_recv (self->pipe->pipe, &buf);
+      rv = sp_client_recv (pipe->pipe, &buf);
       GST_OBJECT_UNLOCK (self);
       if (rv < 0) {
         GST_ELEMENT_ERROR (self, RESOURCE, READ, ("Failed to read from shmsrc"),
             ("Error reading control data: %d", rv));
-        return GST_FLOW_ERROR;
+        goto error;
       }
     }
   } while (buf == NULL);
@@ -367,13 +383,19 @@ gst_shm_src_create (GstPushSrc * psrc, GstBuffer ** outbuf)
 
   gsb = g_slice_new0 (struct GstShmBuffer);
   gsb->buf = buf;
-  gsb->pipe = self->pipe;
-  gst_shm_pipe_inc (self->pipe);
+  gsb->pipe = pipe;
 
   *outbuf = gst_buffer_new_wrapped_full (GST_MEMORY_FLAG_READONLY,
       buf, rv, 0, rv, gsb, free_buffer);
 
   return GST_FLOW_OK;
+
+error:
+  gst_shm_pipe_dec (pipe);
+  return GST_FLOW_ERROR;
+flushing:
+  gst_shm_pipe_dec (pipe);
+  return GST_FLOW_FLUSHING;
 }
 
 static GstStateChangeReturn
@@ -384,9 +406,10 @@ gst_shm_src_change_state (GstElement * element, GstStateChange transition)
 
   switch (transition) {
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
-      if (gst_base_src_is_live (GST_BASE_SRC (element)))
+      if (gst_base_src_is_live (GST_BASE_SRC (element))) {
         if (!gst_shm_src_start_reading (self))
           return GST_STATE_CHANGE_FAILURE;
+      }
     default:
       break;
   }
@@ -397,8 +420,10 @@ gst_shm_src_change_state (GstElement * element, GstStateChange transition)
 
   switch (transition) {
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
-      if (gst_base_src_is_live (GST_BASE_SRC (element)))
+      if (gst_base_src_is_live (GST_BASE_SRC (element))) {
+        gst_shm_src_unlock (GST_BASE_SRC (element));
         gst_shm_src_stop_reading (self);
+      }
     default:
       break;
   }
@@ -428,18 +453,6 @@ gst_shm_src_unlock_stop (GstBaseSrc * bsrc)
   return TRUE;
 }
 
-static void
-gst_shm_pipe_inc (GstShmPipe * pipe)
-{
-  g_return_if_fail (pipe);
-  g_return_if_fail (pipe->src);
-  g_return_if_fail (pipe->use_count > 0);
-
-  GST_OBJECT_LOCK (pipe->src);
-  pipe->use_count++;
-  GST_OBJECT_UNLOCK (pipe->src);
-}
-
 static void
 gst_shm_pipe_dec (GstShmPipe * pipe)
 {
@@ -457,6 +470,10 @@ gst_shm_pipe_dec (GstShmPipe * pipe)
 
   if (pipe->pipe)
     sp_client_close (pipe->pipe);
+
+  gst_poll_remove_fd (pipe->src->poll, &pipe->src->pollfd);
+  gst_poll_fd_init (&pipe->src->pollfd);
+
   GST_OBJECT_UNLOCK (pipe->src);
 
   gst_object_unref (pipe->src);
