@@ -141,7 +141,8 @@ gst_wasapi_sink_class_init (GstWasapiSinkClass * klass)
   g_object_class_install_property (gobject_class,
       PROP_AUDIOCLIENT3,
       g_param_spec_boolean ("use-audioclient3", "Use the AudioClient3 API",
-          "Use the Windows 10 AudioClient3 API when available",
+          "Use the Windows 10 AudioClient3 API when available and if the "
+          "low-latency property is set to TRUE",
           DEFAULT_AUDIOCLIENT3, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gst_element_class_add_static_pad_template (gstelement_class, &sink_template);
@@ -174,6 +175,7 @@ gst_wasapi_sink_init (GstWasapiSink * self)
   self->low_latency = DEFAULT_LOW_LATENCY;
   self->try_audioclient3 = DEFAULT_AUDIOCLIENT3;
   self->event_handle = CreateEvent (NULL, FALSE, FALSE, NULL);
+  self->client_needs_restart = FALSE;
 
   CoInitialize (NULL);
 }
@@ -206,9 +208,8 @@ gst_wasapi_sink_finalize (GObject * object)
 {
   GstWasapiSink *self = GST_WASAPI_SINK (object);
 
-  g_clear_pointer (&self->mix_format, CoTaskMemFree);
-
-  CoUninitialize ();
+  CoTaskMemFree (self->mix_format);
+  self->mix_format = NULL;
 
   if (self->cached_caps != NULL) {
     gst_caps_unref (self->cached_caps);
@@ -219,6 +220,8 @@ gst_wasapi_sink_finalize (GObject * object)
   g_clear_pointer (&self->device_strid, g_free);
   self->mute = FALSE;
 
+  CoUninitialize ();
+
   G_OBJECT_CLASS (gst_wasapi_sink_parent_class)->finalize (object);
 }
 
@@ -295,10 +298,30 @@ gst_wasapi_sink_get_property (GObject * object, guint prop_id,
 static gboolean
 gst_wasapi_sink_can_audioclient3 (GstWasapiSink * self)
 {
-  if (self->sharemode == AUDCLNT_SHAREMODE_SHARED &&
-      self->try_audioclient3 && gst_wasapi_util_have_audioclient3 ())
-    return TRUE;
-  return FALSE;
+  /* AudioClient3 API only makes sense in shared mode */
+  if (self->sharemode != AUDCLNT_SHAREMODE_SHARED)
+    return FALSE;
+
+  if (!self->try_audioclient3) {
+    GST_INFO_OBJECT (self, "AudioClient3 disabled by user");
+    return FALSE;
+  }
+
+  if (!gst_wasapi_util_have_audioclient3 ()) {
+    GST_INFO_OBJECT (self, "AudioClient3 not available on this OS");
+    return FALSE;
+  }
+
+  /* Only use audioclient3 when low-latency is requested because otherwise
+   * very slow machines and VMs with 1 CPU allocated will get glitches:
+   * https://bugzilla.gnome.org/show_bug.cgi?id=794497 */
+  if (!self->low_latency) {
+    GST_INFO_OBJECT (self, "AudioClient3 disabled because low-latency mode "
+        "was not requested");
+    return FALSE;
+  }
+
+  return TRUE;
 }
 
 static GstCaps *
@@ -318,22 +341,26 @@ gst_wasapi_sink_get_caps (GstBaseSink * bsink, GstCaps * filter)
 
     template_caps = gst_pad_get_pad_template_caps (bsink->sinkpad);
 
-    if (!self->client)
-      gst_wasapi_sink_open (GST_AUDIO_SINK (bsink));
+    if (!self->client) {
+      caps = template_caps;
+      goto out;
+    }
 
     ret = gst_wasapi_util_get_device_format (GST_ELEMENT (self),
         self->sharemode, self->device, self->client, &format);
     if (!ret) {
       GST_ELEMENT_ERROR (self, STREAM, FORMAT, (NULL),
           ("failed to detect format"));
-      goto out;
+      gst_caps_unref (template_caps);
+      return NULL;
     }
 
     gst_wasapi_util_parse_waveformatex ((WAVEFORMATEXTENSIBLE *) format,
         template_caps, &caps, &self->positions);
     if (caps == NULL) {
       GST_ELEMENT_ERROR (self, STREAM, FORMAT, (NULL), ("unknown format"));
-      goto out;
+      gst_caps_unref (template_caps);
+      return NULL;
     }
 
     {
@@ -355,9 +382,8 @@ gst_wasapi_sink_get_caps (GstBaseSink * bsink, GstCaps * filter)
     caps = filtered;
   }
 
-  GST_DEBUG_OBJECT (self, "returning caps %" GST_PTR_FORMAT, caps);
-
 out:
+  GST_DEBUG_OBJECT (self, "returning caps %" GST_PTR_FORMAT, caps);
   return caps;
 }
 
@@ -432,7 +458,7 @@ gst_wasapi_sink_get_can_frames (GstWasapiSink * self)
 
   /* Frames the card hasn't rendered yet */
   hr = IAudioClient_GetCurrentPadding (self->client, &n_frames_padding);
-  HR_FAILED_RET (hr, IAudioClient::GetCurrentPadding, -1);
+  HR_FAILED_ELEMENT_ERROR_RET (hr, IAudioClient::GetCurrentPadding, self, -1);
 
   GST_DEBUG_OBJECT (self, "%i unread frames (padding)", n_frames_padding);
 
@@ -449,6 +475,8 @@ gst_wasapi_sink_prepare (GstAudioSink * asink, GstAudioRingBufferSpec * spec)
   guint bpf, rate, devicep_frames;
   HRESULT hr;
 
+  CoInitialize (NULL);
+
   if (gst_wasapi_sink_can_audioclient3 (self)) {
     if (!gst_wasapi_util_initialize_audioclient3 (GST_ELEMENT (self), spec,
             (IAudioClient3 *) self->client, self->mix_format, self->low_latency,
@@ -533,9 +561,6 @@ gst_wasapi_sink_prepare (GstAudioSink * asink, GstAudioRingBufferSpec * spec)
   gst_audio_ring_buffer_set_channel_positions (GST_AUDIO_BASE_SINK
       (self)->ringbuffer, self->positions);
 
-  /* Increase the thread priority to reduce glitches */
-  self->thread_priority_handle = gst_wasapi_util_set_thread_characteristics ();
-
   res = TRUE;
 
 beach:
@@ -552,15 +577,7 @@ gst_wasapi_sink_unprepare (GstAudioSink * asink)
 {
   GstWasapiSink *self = GST_WASAPI_SINK (asink);
 
-  if (self->sharemode == AUDCLNT_SHAREMODE_EXCLUSIVE &&
-      !gst_wasapi_sink_can_audioclient3 (self))
-    CoUninitialize ();
-
-  if (self->thread_priority_handle != NULL) {
-    gst_wasapi_util_revert_thread_characteristics
-        (self->thread_priority_handle);
-    self->thread_priority_handle = NULL;
-  }
+  CoUninitialize ();
 
   if (self->client != NULL) {
     IAudioClient_Stop (self->client);
@@ -580,42 +597,95 @@ gst_wasapi_sink_write (GstAudioSink * asink, gpointer data, guint length)
   GstWasapiSink *self = GST_WASAPI_SINK (asink);
   HRESULT hr;
   gint16 *dst = NULL;
-  guint pending = length;
+  DWORD dwWaitResult;
+  guint can_frames, have_frames, n_frames, write_len, written_len = 0;
 
-  while (pending > 0) {
-    guint can_frames, have_frames, n_frames, write_len;
+  GST_OBJECT_LOCK (self);
+  if (self->client_needs_restart) {
+    hr = IAudioClient_Start (self->client);
+    HR_FAILED_ELEMENT_ERROR_AND (hr, IAudioClient::Start, self,
+        GST_OBJECT_UNLOCK (self); goto err);
+    self->client_needs_restart = FALSE;
+  }
+  GST_OBJECT_UNLOCK (self);
 
-    WaitForSingleObject (self->event_handle, INFINITE);
+  /* We have N frames to be written out */
+  have_frames = length / (self->mix_format->nBlockAlign);
+
+  if (self->sharemode == AUDCLNT_SHAREMODE_EXCLUSIVE) {
+    /* In exlusive mode we have to wait always */
+    dwWaitResult = WaitForSingleObject (self->event_handle, INFINITE);
+    if (dwWaitResult != WAIT_OBJECT_0) {
+      GST_ERROR_OBJECT (self, "Error waiting for event handle: %x",
+          (guint) dwWaitResult);
+      goto err;
+    }
 
-    /* We have N frames to be written out */
-    have_frames = pending / (self->mix_format->nBlockAlign);
-    /* We have can_frames space in the output buffer */
     can_frames = gst_wasapi_sink_get_can_frames (self);
-    /* We will write out these many frames, and this much length */
-    n_frames = MIN (can_frames, have_frames);
-    write_len = n_frames * self->mix_format->nBlockAlign;
+    if (can_frames < 0) {
+      GST_ERROR_OBJECT (self, "Error getting frames to write to");
+      goto err;
+    }
+    /* In exclusive mode we need to fill the whole buffer in one go or
+     * GetBuffer will error out */
+    if (can_frames != have_frames) {
+      GST_ERROR_OBJECT (self,
+          "Need at %i frames to write for exclusive mode, but got %i",
+          can_frames, have_frames);
+      goto err;
+    }
+  } else {
+    /* In shared mode we can write parts of the buffer, so only wait
+     * in case we can't write anything */
+    can_frames = gst_wasapi_sink_get_can_frames (self);
+    if (can_frames < 0) {
+      GST_ERROR_OBJECT (self, "Error getting frames to write to");
+      goto err;
+    }
 
-    GST_DEBUG_OBJECT (self, "total: %i, have_frames: %i (%i bytes), "
-        "can_frames: %i, will write: %i (%i bytes)", self->buffer_frame_count,
-        have_frames, pending, can_frames, n_frames, write_len);
-
-    hr = IAudioRenderClient_GetBuffer (self->render_client, n_frames,
-        (BYTE **) & dst);
-    HR_FAILED_AND (hr, IAudioRenderClient::GetBuffer, length = 0; goto beach);
-
-    memcpy (dst, data, write_len);
-
-    hr = IAudioRenderClient_ReleaseBuffer (self->render_client, n_frames,
-        self->mute ? AUDCLNT_BUFFERFLAGS_SILENT : 0);
-    HR_FAILED_AND (hr, IAudioRenderClient::ReleaseBuffer, length = 0;
-        goto beach);
-
-    pending -= write_len;
+    if (can_frames == 0) {
+      dwWaitResult = WaitForSingleObject (self->event_handle, INFINITE);
+      if (dwWaitResult != WAIT_OBJECT_0) {
+        GST_ERROR_OBJECT (self, "Error waiting for event handle: %x",
+            (guint) dwWaitResult);
+        goto err;
+      }
+      can_frames = gst_wasapi_sink_get_can_frames (self);
+      if (can_frames < 0) {
+        GST_ERROR_OBJECT (self, "Error getting frames to write to");
+        goto err;
+      }
+    }
   }
 
-beach:
+  /* We will write out these many frames, and this much length */
+  n_frames = MIN (can_frames, have_frames);
+  write_len = n_frames * self->mix_format->nBlockAlign;
 
-  return length;
+  GST_DEBUG_OBJECT (self, "total: %i, have_frames: %i (%i bytes), "
+      "can_frames: %i, will write: %i (%i bytes)", self->buffer_frame_count,
+      have_frames, length, can_frames, n_frames, write_len);
+
+  hr = IAudioRenderClient_GetBuffer (self->render_client, n_frames,
+      (BYTE **) & dst);
+  HR_FAILED_ELEMENT_ERROR_AND (hr, IAudioRenderClient::GetBuffer, self,
+      goto err);
+
+  memcpy (dst, data, write_len);
+
+  hr = IAudioRenderClient_ReleaseBuffer (self->render_client, n_frames,
+      self->mute ? AUDCLNT_BUFFERFLAGS_SILENT : 0);
+  HR_FAILED_ELEMENT_ERROR_AND (hr, IAudioRenderClient::ReleaseBuffer, self,
+      goto err);
+
+  written_len = write_len;
+
+out:
+  return written_len;
+
+err:
+  written_len = -1;
+  goto out;
 }
 
 static guint
@@ -637,12 +707,18 @@ gst_wasapi_sink_reset (GstAudioSink * asink)
   GstWasapiSink *self = GST_WASAPI_SINK (asink);
   HRESULT hr;
 
+  GST_INFO_OBJECT (self, "reset called");
+
   if (!self->client)
     return;
 
+  GST_OBJECT_LOCK (self);
   hr = IAudioClient_Stop (self->client);
-  HR_FAILED_RET (hr, IAudioClient::Stop,);
+  HR_FAILED_AND (hr, IAudioClient::Stop,);
 
   hr = IAudioClient_Reset (self->client);
-  HR_FAILED_RET (hr, IAudioClient::Reset,);
+  HR_FAILED_AND (hr, IAudioClient::Reset,);
+
+  self->client_needs_restart = TRUE;
+  GST_OBJECT_UNLOCK (self);
 }
