@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008 Ole Andr Vadla Ravns <ole.andre.ravnas@tandberg.com>
+ * Copyright (C) 2008 Ole André Vadla Ravnås <ole.andre.ravnas@tandberg.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -22,6 +22,8 @@
 /* This plugin is from the era of Windows XP and uses APIs that have been
  * deprecated since then. Let's pretend we're Windows XP too so that Windows
  * lets us use that deprecated API. */
+#undef NTDDI_VERSION
+#undef _WIN32_WINNT
 #define NTDDI_VERSION NTDDI_WINXP
 #define _WIN32_WINNT _WIN32_WINNT_WINXP
 
