@@ -355,7 +355,7 @@ GST_START_TEST (test_caps_query)
   gst_caps_unref (caps);
   gst_caps_unref (restriction_caps);
 
-  /* check that compositor proxies downstream interlaced-mode */
+  /* check that compositor proxies downstream interlace-mode */
   restriction_caps =
       gst_caps_from_string ("video/x-raw, interlace-mode=(string)interleaved");
   g_object_set (capsfilter, "caps", restriction_caps, NULL);
@@ -385,6 +385,7 @@ GST_START_TEST (test_caps_query_interlaced)
   GstCaps *caps;
   GstCaps *caps_mixed, *caps_progressive, *caps_interleaved;
   GstEvent *caps_event;
+  GstQuery *drain;
 
   caps_interleaved =
       gst_caps_from_string ("video/x-raw, interlace-mode=interleaved");
@@ -412,7 +413,7 @@ GST_START_TEST (test_caps_query_interlaced)
   fail_unless (gst_caps_can_intersect (caps, caps_mixed));
   gst_caps_unref (caps);
 
-  /* now set caps on the pad, it should restrict the interlaced-mode for
+  /* now set caps on the pad, it should restrict the interlace-mode for
    * future caps */
   caps = gst_caps_from_string ("video/x-raw, width=100, height=100, "
       "format=RGB, framerate=1/1, interlace-mode=progressive");
@@ -420,6 +421,11 @@ GST_START_TEST (test_caps_query_interlaced)
   gst_caps_unref (caps);
   fail_unless (gst_pad_send_event (sinkpad, caps_event));
 
+  /* Send drain query to make sure this is processed */
+  drain = gst_query_new_drain ();
+  gst_pad_query (sinkpad, drain);
+  gst_query_unref (drain);
+
   /* now recheck the interlace-mode */
   gst_object_unref (sinkpad);
   sinkpad = gst_element_get_request_pad (compositor, "sink_%u");
@@ -443,8 +449,9 @@ static void
 add_interlaced_mode_to_caps (GstCaps * caps, const gchar * mode)
 {
   GstStructure *s;
+  gint i;
 
-  for (gint i = 0; i < gst_caps_get_size (caps); i++) {
+  for (i = 0; i < gst_caps_get_size (caps); i++) {
     s = gst_caps_get_structure (caps, i);
     gst_structure_set (s, "interlace-mode", G_TYPE_STRING, mode, NULL);
   }
@@ -564,7 +571,6 @@ run_late_caps_set_test (GstCaps * first_caps, GstCaps * expected_query_caps,
   GstStateChangeReturn state_res;
   GstPad *sinkpad_2;
   GstCaps *caps;
-  GstEvent *caps_event;
   GstBus *bus;
   GstMessage *msg;
 
@@ -595,8 +601,9 @@ run_late_caps_set_test (GstCaps * first_caps, GstCaps * expected_query_caps,
   sinkpad_2 = gst_element_get_request_pad (compositor, "sink_%u");
   caps = gst_pad_query_caps (sinkpad_2, NULL);
   fail_unless (gst_caps_is_subset (expected_query_caps, caps));
-  caps_event = gst_event_new_caps (second_caps);
-  fail_unless (gst_pad_send_event (sinkpad_2, caps_event) == accept_caps);
+  gst_caps_unref (caps);
+  caps = gst_pad_query_caps (sinkpad_2, second_caps);
+  fail_unless (gst_caps_is_empty (caps) != accept_caps);
   gst_caps_unref (caps);
   gst_object_unref (sinkpad_2);
 
